from .. import Provider as LoremProvider


class Provider(LoremProvider):
    """Implement lorem provider for ``he_IL`` locale."""

    word_list = (
        "אאוגו", "אגת", "אדיפיסינג", "אדנדום", "אט", "איאקוליס", "איבן", "איף",
        "איפסום", "אלית", "אלמנקום", "אמט", "אס", "אפאח", "אקווזמן", "ארווס",
        "בגורמי", "בורק?", "בלובק", "בלינדו", "בלינך", "בליקרה", "בעליק",
        "בעריר", "בראיט", "ברומץ", "בריקנה", "ברשג", "גדדיש", "גולר", "גק",
        "דול", "דולור", "דומור", "דז", "דיאם", "דלאמת", "דס", "הבקיץ", "הדש",
        "הועניב", "היושבב", "הכייר", "השמה", "התידם", "וואל", "וולופטה",
        "וחאית", "ולחת", "ולתיעם", "ומעיוט", "ומרגשח", "וסטיבולום", "וסתעד",
        "וק", "ותלברו", "זותה", "חשלו", "טידום", "יבש", "יהול", "ישבעס", "כאנה",
        "כלרשט", "להאמית", "לורם", "ליאמום", "ליבם", "ליץ", "לכימפו", "לכנו",
        "לכנוץ", "למטכין", "למרקוח", "למרקל", "לפמעט", "לפריקך", "לפתיעם",
        "לקטוס", "לרטי", "לתיג", "לתכי", "מא", "מגמש", "מונחף", "מונפרד",
        "מונפרר", "מוסן", "מורגם", "מיחוצים", "מנורך", "מנכם", "מנק", "מנת",
        "מרגשי", "נובש", "נולום", "נון", "נונסטי", "ניבאה", "ניסי", "ניצאחו",
        "נמרגי", "נשואי", "סאפיאן", "סוברט", "סולגק", "סוליסי", "סחטיר",
        "סטום", "סיט", "סילקוף", "סכעיט", "סת", "סתשם", "עמחליף", "ערששף",
        "פוסיליס", "צוט", "צופעט", "צורק", "קוויז", "קוויס", "קולהע", "קולורס",
        "קונדימנטום", "קונסקטורר", "קורוס", "קלאצי", "קלובר", "קראס", "קרהשק",
        "רוגצה", "שבצק", "שהכים", "שלושע", "שמחויט", "שנרא", "שעותלשך", "שערש",
        "תוק", "תצטנפל", "תצטריק", "תרבנך",
    )



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats = (
        '{{first_name}} {{last_name}}',
    )

    # First names are from https://www.babycenter.in/a25010193/modern-indian-baby-names
    # Last names are from https://www.familyeducation.com/baby-names/browse-origin/surname/indian

    first_names = (
        'Aarav',
        'Aarush',
        'Aayush',
        'Abram',
        'Advik',
        'Akarsh',
        'Anay',
        'Aniruddh',
        'Arhaan',
        'Armaan',
        'Arnav',
        'Azad',
        'Badal',
        'Bhavin',
        'Chirag',
        'Darshit',
        'Devansh',
        'Dhanuk',
        'Dhruv',
        'Divij',
        'Divit',
        'Divyansh',
        'Ehsaan',
        'Emir',
        'Faiyaz',
        'Farhan',
        'Fateh',
        'Gatik',
        'Gokul',
        'Hansh',
        'Himmat',
        'Hiran',
        'Hridaan',
        'Hunar',
        'Indrajit',
        'Indranil',
        'Ishaan',
        'Ivan',
        'Jayant',
        'Jayesh',
        'Jivin',
        'Kabir',
        'Kanav',
        'Kartik',
        'Kiaan',
        'Krish',
        'Lagan',
        'Lakshay',
        'Lakshit',
        'Madhav',
        'Madhup',
        'Manikya',
        'Mehul',
        'Miraan',
        'Nakul',
        'Nirvaan',
        'Nishith',
        'Ojas',
        'Onkar',
        'Pranay',
        'Prerak',
        'Priyansh',
        'Purab',
        'Raghav',
        'Ranbir',
        'Raunak',
        'Reyansh',
        'Riaan',
        'Ritvik',
        'Rohan',
        'Romil',
        'Ryan',
        'Sahil',
        'Saksham',
        'Samar',
        'Samarth',
        'Shaan',
        'Shalv',
        'Shamik',
        'Shayak',
        'Shlok',
        'Shray',
        'Stuvan',
        'Sumer',
        'Taimur',
        'Taran',
        'Tejas',
        'Tushar',
        'Umang',
        'Uthkarsh',
        'Vaibhav',
        'Veer',
        'Vidur',
        'Vihaan',
        'Vivaan',
        'Yakshit',
        'Yuvaan',
        'Yuvraj ',
        'Zain',
        'Zeeshan',
        'Aaina',
        'Aarna',
        'Aaryahi',
        'Adah',
        'Adira',
        'Advika',
        'Ahana ',
        'Alia',
        'Alisha',
        'Amani',
        'Amira',
        'Anahi',
        'Anahita',
        'Anaya',
        'Anika',
        'Anvi',
        'Anya',
        'Aradhya',
        'Ayesha',
        'Bhamini',
        'Charvi',
        'Damini',
        'Dishani',
        'Diya',
        'Drishya',
        'Ela',
        'Elakshi',
        'Eshani',
        'Eva',
        'Hazel',
        'Heer',
        'Hrishita',
        'Inaaya ',
        'Ira',
        'Ishita',
        'Ivana',
        'Jhanvi',
        'Jivika',
        'Jiya',
        'Kaira',
        'Kashvi',
        'Kavya',
        'Keya',
        'Khushi',
        'Kiara',
        'Kimaya',
        'Kismat',
        'Lavanya',
        'Mahika',
        'Manjari',
        'Mannat',
        'Miraya',
        'Misha',
        'Mishti',
        'Myra',
        'Navya',
        'Nayantara',
        'Neelofar',
        'Nehmat',
        'Neysa',
        'Nirvi',
        'Nitara',
        'Nitya',
        'Oorja',
        'Pari',
        'Parinaaz',
        'Pihu',
        'Piya',
        'Prisha',
        'Rania',
        'Rasha',
        'Rati',
        'Renee',
        'Rhea',
        'Riya',
        'Saanvi',
        'Saira',
        'Samaira',
        'Samiha',
        'Sana',
        'Sara',
        'Seher',
        'Shanaya',
        'Siya',
        'Suhana',
        'Tanya',
        'Tara',
        'Tarini',
        'Tiya',
        'Trisha',
        'Urvi',
        'Vanya',
        'Vardaniya',
        'Vedika',
        'Vritika',
        'Yashvi',
        'Yasmin',
        'Zaina',
        'Zara',
        'Zoya',
    )

    last_names = (
        'Acharya',
        'Agarwal',
        'Agate',
        'Aggarwal',
        'Agrawal',
        'Ahluwalia',
        'Ahuja',
        'Amble',
        'Anand',
        'Andra',
        'Anne',
        'Apte',
        'Arora',
        'Arya',
        'Atwal',
        'Aurora',
        'Babu',
        'Badal',
        'Badami',
        'Bahl',
        'Bahri',
        'Bail',
        'Bains',
        'Bajaj',
        'Bajwa',
        'Bakshi',
        'Bal',
        'Bala',
        'Bala',
        'Balakrishnan',
        'Balan',
        'Balasubramanian',
        'Balay',
        'Bali',
        'Bandi',
        'Banerjee',
        'Banik',
        'Bansal',
        'Barad',
        'Barad',
        'Baral',
        'Baria',
        'Barman',
        'Basak',
        'Bassi',
        'Basu',
        'Bath',
        'Batra',
        'Batta',
        'Bava',
        'Bawa',
        'Bedi',
        'Behl',
        'Ben',
        'Bera',
        'Bhagat',
        'Bhakta',
        'Bhalla',
        'Bhandari',
        'Bhardwaj',
        'Bhargava',
        'Bhasin',
        'Bhat',
        'Bhatia',
        'Bhatnagar',
        'Bhatt',
        'Bhattacharyya',
        'Bhatti',
        'Bhavsar',
        'Bir',
        'Biswas',
        'Boase',
        'Bobal',
        'Bora',
        'Bora',
        'Borah',
        'Borde',
        'Borra',
        'Bose',
        'Brahmbhatt',
        'Brar',
        'Buch',
        'Buch',
        'Bumb',
        'Butala',
        'Chacko',
        'Chad',
        'Chada',
        'Chadha',
        'Chahal',
        'Chakrabarti',
        'Chakraborty',
        'Chana',
        'Chand',
        'Chanda',
        'Chander',
        'Chandra',
        'Chandran',
        'Char',
        'Chatterjee',
        'Chaudhari',
        'Chaudhary',
        'Chaudhry',
        'Chaudhuri',
        'Chaudry',
        'Chauhan',
        'Chawla',
        'Cheema',
        'Cherian',
        'Chhabra',
        'Chokshi',
        'Chopra',
        'Choudhary',
        'Choudhry',
        'Choudhury',
        'Chowdhury',
        'Comar',
        'Contractor',
        'D’Alia',
        'Dada',
        'Dalal',
        'Dani',
        'Dar',
        'Dara',
        'Dara',
        'Das',
        'Dasgupta',
        'Dash',
        'Dass',
        'Date',
        'Datta',
        'Dave',
        'Dayal',
        'De',
        'Deep',
        'Deo',
        'Deol',
        'Desai',
        'Deshmukh',
        'Deshpande',
        'Devan',
        'Devi',
        'Dewan',
        'Dey',
        'Dhaliwal',
        'Dhar',
        'Dhar',
        'Dhawan',
        'Dhillon',
        'Dhingra',
        'Din',
        'Divan',
        'Dixit',
        'Doctor',
        'Dora',
        'Doshi',
        'Dua',
        'Dube',
        'Dubey',
        'Dugal',
        'Dugar',
        'Dugar',
        'Dutt',
        'Dutta',
        'Dyal',
        'Edwin',
        'Gaba',
        'Gade',
        'Gala',
        'Gandhi',
        'Ganesan',
        'Ganesh',
        'Ganguly',
        'Gara',
        'Garde',
        'Garg',
        'Gera',
        'Ghose',
        'Ghosh',
        'Gill',
        'Goda',
        'Goel',
        'Gokhale',
        'Gola',
        'Gole',
        'Golla',
        'Gopal',
        'Goswami',
        'Gour',
        'Goyal',
        'Grewal',
        'Grover',
        'Guha',
        'Gulati',
        'Gupta',
        'Halder',
        'Handa',
        'Hans',
        'Hari',
        'Hayer',
        'Hayre',
        'Hegde',
        'Hora',
        'Issac',
        'Iyengar',
        'Iyer',
        'Jaggi',
        'Jain',
        'Jani',
        'Jayaraman',
        'Jha',
        'Jhaveri',
        'Johal',
        'Joshi',
        'Kadakia',
        'Kade',
        'Kakar',
        'Kala',
        'Kala',
        'Kala',
        'Kale',
        'Kalita',
        'Kalla',
        'Kamdar',
        'Kanda',
        'Kannan',
        'Kant',
        'Kapadia',
        'Kapoor',
        'Kapur',
        'Kar',
        'Kara',
        'Karan',
        'Kari',
        'Karnik',
        'Karpe',
        'Kashyap',
        'Kata',
        'Kaul',
        'Kaur',
        'Keer',
        'Keer',
        'Khalsa',
        'Khanna',
        'Khare',
        'Khatri',
        'Khosla',
        'Khurana',
        'Kibe',
        'Kohli',
        'Konda',
        'Korpal',
        'Koshy',
        'Kota',
        'Kothari',
        'Krish',
        'Krishna',
        'Krishnamurthy',
        'Krishnan',
        'Kulkarni',
        'Kumar',
        'Kumer',
        'Kunda',
        'Kurian',
        'Kuruvilla',
        'Lad',
        'Lad',
        'Lal',
        'Lala',
        'Lall',
        'Lalla',
        'Lanka',
        'Lata',
        'Loke',
        'Loyal',
        'Luthra',
        'Madan',
        'Madan',
        'Magar',
        'Mahajan',
        'Mahal',
        'Maharaj',
        'Majumdar',
        'Malhotra',
        'Mall',
        'Mallick',
        'Mammen',
        'Mand',
        'Manda',
        'Mandal',
        'Mander',
        'Mane',
        'Mangal',
        'Mangat',
        'Mani',
        'Mani',
        'Mann',
        'Mannan',
        'Manne',
        'Master',
        'Raj',
        'Raja',
        'Rajagopal',
        'Rajagopalan',
        'Rajan',
        'Raju',
        'Ram',
        'Rama',
        'Ramachandran',
        'Ramakrishnan',
        'Raman',
        'Ramanathan',
        'Ramaswamy',
        'Ramesh',
        'Rana',
        'Randhawa',
        'Ranganathan',
        'Rao',
        'Rastogi',
        'Ratta',
        'Rattan',
        'Ratti',
        'Rau',
        'Raval',
        'Ravel',
        'Ravi',
        'Ray',
        'Reddy',
        'Rege',
        'Rout',
        'Roy',
        'Sabharwal',
        'Sachar',
        'Sachdev',
        'Sachdeva',
        'Sagar',
        'Saha',
        'Sahni',
        'Sahota',
        'Saini',
        'Salvi',
        'Sama',
        'Sami',
        'Sampath',
        'Samra',
        'Sandal',
        'Sandhu',
        'Sane',
        'Sangha',
        'Sanghvi',
        'Sani',
        'Sankar',
        'Sankaran',
        'Sant',
        'Saraf',
        'Saran',
        'Sarin',
        'Sarkar',
        'Sarma',
        'Sarna',
        'Sarraf',
        'Sastry',
        'Sathe',
        'Savant',
        'Sawhney',
        'Saxena',
        'Sehgal',
        'Sekhon',
        'Sem',
        'Sen',
        'Sengupta',
        'Seshadri',
        'Seth',
        'Sethi',
        'Setty',
        'Sha',
        'Shah',
        'Shan',
        'Shankar',
        'Shanker',
        'Sharaf',
        'Sharma',
        'Shenoy',
        'Shere',
        'Sheth',
        'Shetty',
        'Shroff',
        'Shukla',
        'Sibal',
        'Sidhu',
        'Singh',
        'Singhal',
        'Sinha',
        'Sodhi',
        'Solanki',
        'Som',
        'Soman',
        'Soni',
        'Sood',
        'Sridhar',
        'Srinivas',
        'Srinivasan',
        'Srivastava',
        'Subramaniam',
        'Subramanian',
        'Sule',
        'Sundaram',
        'Sunder',
        'Sur',
        'Sura',
        'Suresh',
        'Suri',
        'Swaminathan',
        'Swamy',
        'Tailor',
        'Tak',
        'Talwar',
        'Tandon',
        'Taneja',
        'Tank',
        'Tara',
        'Tata',
        'Tella',
        'Thaker',
        'Thakkar',
        'Thakur',
        'Thaman',
        'Tiwari',
        'Toor',
        'Tripathi',
        'Trivedi',
        'Upadhyay',
        'Uppal',
        'Vaidya',
        'Vala',
        'Varghese',
        'Varkey',
        'Varma',
        'Varty',
        'Varughese',
        'Vasa',
        'Venkataraman',
        'Venkatesh',
        'Verma',
        'Vig',
        'Virk',
        'Viswanathan',
        'Vohra',
        'Vora',
        'Vyas',
        'Wable',
        'Wadhwa',
        'Wagle',
        'Wali',
        'Wali',
        'Walia',
        'Walla',
        'Warrior',
        'Wason',
        'Yadav',
        'Yogi',
        'Yohannan',
        'Zacharia',
        'Zachariah',
    )



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats_male = (
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{last_name}}',
        '{{first_name_male}} {{prefix}} {{last_name}}',
        '{{first_name_male}} {{last_name}}-{{last_name}}',
    )

    formats_female = (
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{last_name}}',
        '{{first_name_female}} {{prefix}} {{last_name}}',
        '{{first_name_female}} {{last_name}}-{{last_name}}',
        '{{first_name_female}}-{{first_name_female}} {{last_name}}',
    )

    formats = formats_male + formats_female

    first_names_male = (
        'Afonso', 'Alexandre', 'Álvaro', 'André', 'Ângelo', 'António', 'Artur',
        'Benjamim', 'Bernardo', 'Brian', 'Bruno', 'Bryan', 'Carlos',
        'Cláudio', 'Cristiano', 'César', 'Daniel', 'David', 'Denis',
        'Diego', 'Dinis', 'Diogo', 'Duarte', 'Edgar', 'Eduardo', 'Emanuel',
        'Enzo', 'Fernando', 'Filipe', 'Francisco', 'Frederico', 'Fábio',
        'Gabriel', 'Gaspar', 'Gil', 'Gonçalo', 'Guilherme', 'Gustavo',
        'Henrique', 'Hugo', 'Igor', 'Isaac', 'Ismael', 'Ivan', 'Ivo', 'Jaime',
        'Joaquim', 'Joel', 'Jorge', 'José', 'João', 'Kevin', 'Kévim',
        'Leandro', 'Leonardo', 'Lisandro', 'Lourenço', 'Luca', 'Lucas',
        'Luís', 'Manuel', 'Marco', 'Marcos', 'Martim', 'Mateus',
        'Matias', 'Mauro', 'Micael', 'Miguel', 'Márcio', 'Mário',
        'Nelson', 'Noa', 'Noah', 'Nuno', 'Paulo', 'Pedro', 'Rafael',
        'Renato', 'Ricardo', 'Rodrigo', 'Rui', 'Rúben', 'Salvador',
        'Samuel', 'Sandro', 'Santiago', 'Sebastião', 'Simão', 'Sérgio', 'Tiago',
        'Tomás', 'Tomé', 'Valentim', 'Vasco', 'Vicente', 'Vítor', 'William',
        'Wilson', 'Xavier',
    )

    first_names_female = (
        'Adriana', 'Alexandra', 'Alice', 'Alícia', 'Amélia', 'Ana', 'Andreia',
        'Ângela', 'Anita', 'Ariana', 'Beatriz', 'Benedita', 'Bianca', 'Bruna',
        'Bárbara', 'Caetana', 'Camila', 'Carlota', 'Carminho', 'Carolina',
        'Catarina', 'Clara', 'Constança', 'Daniela', 'Diana', 'Débora', 'Eduarda',
        'Ema', 'Emma', 'Emília', 'Erica', 'Érica', 'Erika', 'Eva', 'Fabiana',
        'Filipa', 'Flor', 'Francisca', 'Gabriela', 'Helena', 'Iara', 'Inês',
        'Irina', 'Íris', 'Isabel', 'Isabela', 'Joana', 'Juliana', 'Jéssica',
        'Júlia', 'Kelly', 'Kyara', 'Lara', 'Larissa', 'Laura', 'Leonor', 'Letícia',
        'Lia', 'Lorena', 'Luana', 'Luciana', 'Luna', 'Luísa', 'Lúcia', 'Madalena',
        'Mafalda', 'Mara', 'Margarida', 'Maria', 'Mariana', 'Marta', 'Matilde',
        'Melissa', 'Mia', 'Miriam', 'Mélanie', 'Naiara', 'Nair', 'Nicole', 'Nádia',
        'Núria', 'Patrícia', 'Petra', 'Pilar', 'Rafaela', 'Raquel', 'Renata',
        'Rita', 'Salomé', 'Sara', 'Sofia', 'Soraia', 'Tatiana', 'Teresa',
        'Valentina', 'Vera', 'Victória', 'Violeta', 'Vitória', 'Yara', 'Yasmin',
    )

    first_names = first_names_male + first_names_female

    last_names = (
        'Abreu', 'Almeida', 'Alves', 'Amaral', 'Amorim', 'Andrade', 'Anjos',
        'Antunes', 'Araújo', 'Assunção', 'Azevedo', 'Baptista', 'Barbosa',
        'Barros', 'Batista', 'Borges', 'Branco', 'Brito', 'Campos', 'Cardoso',
        'Carneiro', 'Carvalho', 'Castro', 'Coelho', 'Correia', 'Costa', 'Cruz',
        'Cunha', 'Domingues', 'Esteves', 'Faria', 'Fernandes', 'Ferreira',
        'Figueiredo', 'Fonseca', 'Freitas', 'Garcia', 'Gaspar', 'Gomes',
        'Gonçalves', 'Guerreiro', 'Henriques', 'Jesus', 'Leal', 'Leite', 'Lima',
        'Lopes', 'Loureiro', 'Lourenço', 'Macedo', 'Machado', 'Magalhães',
        'Maia', 'Marques', 'Martins', 'Matias', 'Matos', 'Melo', 'Mendes',
        'Miranda', 'Monteiro', 'Morais', 'Moreira', 'Mota', 'Moura',
        'Nascimento', 'Neto', 'Neves', 'Nogueira', 'Nunes', 'Oliveira',
        'Pacheco', 'Paiva', 'Pereira', 'Pinheiro', 'Pinho', 'Pinto', 'Pires',
        'Ramos', 'Reis', 'Ribeiro', 'Rocha', 'Rodrigues', 'Santos', 'Silva',
        'Simões', 'Soares', 'Sousa', 'Sá', 'Tavares', 'Teixeira', 'Torres',
        'Valente', 'Vaz', 'Vicente', 'Vieira',
    )

    prefixes = ('de', 'da', 'do')

    def prefix(self):
        return self.random_element(self.prefixes)



from .. import BaseProvider

localized = True


class Provider(BaseProvider):
    ssn_formats = ("###-##-####",)

    def ssn(self):
        return self.bothify(self.random_element(self.ssn_formats))



from collections import OrderedDict
from string import ascii_uppercase

from .. import Provider as AddressProvider


class Provider(AddressProvider):
    """
    Provider for addresses for en_PH locale

    Like many things in the Philippines, even addresses are more complicated than necessary. This provider is already
    a gross oversimplification, and it is still a lot more complicated VS providers from other locales despite taking
    shortcuts. Below are some tidbits of information that, as a whole, shaped the design decisions of this provider.

    - There are many levels of geopolitical division, thus many levels of local government:
        * There are three major island groups - Luzon, Visayas, Mindanao
        * Those major groups are divided into 17 different regions.
        * Each region is divided into provinces with the exception of the National Capital Region aka Metro Manila.
        * Each province is composed of multiple cities/municipalities.
        * Metro Manila, like a province, is composed of multiple cities/municipalities, but it is a region.
        * Each city/municipality is composed of multiple smaller local government units called barangays.
        * In some places, some barangays are divided further, and as of 2019, there are 42,045 barangays on record.
    - Metro Manila is part of Luzon geographically, but it is almost always treated as a separate entity politically,
      economically, statistically, and so on, since it is home to around 13% of the population despite being only around
      0.2% of the country's total land area.
    - Names of cities, municipalities, and barangays vary a lot. Furthermore, if a place has a non-English name, there
      will almost always be no English translation and vice-versa. It is essentially impossible to generate fake city,
      municipality, and barangay names in a similar manner used in the other "en" locales while being locale specific.
    - Subdivisions and other higher density housing (like high-rise condominiums) are popular in real estate.
    - The 13th floor is omitted in buildings like in many parts of the world.
    - The floor number distribution is partly based on the tallest buildings in the Philippines and partly anecdotal,
      but the general idea is that the higher the floor number is, the lower probability of it appearing. Furthermore,
      as the floor number approaches the highest floors of the tallest buildings, the probability plummets further.
    - The address distribution is based on the official 2015 population census.
    - Addresses should include a barangay, but it has been dropped to keep things sane, all things considered.
    - In addition to numbered floors, buildings have ground floors and may have lower ground, upper ground, mezzanine,
      and basement floors. Buildings may also have units on any of those floors, but the naming scheme varies, so they
      have been dropped, again to keep things sane.

    Sources:
    - https://en.wikipedia.org/wiki/Provinces_of_the_Philippines
    - https://en.wikipedia.org/wiki/List_of_cities_and_municipalities_in_the_Philippines
    - https://en.wikipedia.org/wiki/Barangay
    - https://en.wikipedia.org/wiki/Postal_addresses_in_the_Philippines
    - https://en.wikipedia.org/wiki/List_of_ZIP_codes_in_the_Philippines
    - https://www.phlpost.gov.ph/
    - http://en.wikipedia.org/wiki/List_of_tallest_buildings_in_the_Philippines
    - https://psa.gov.ph/sites/default/files/attachments/hsd/pressrelease/2015%20population%20counts%20Summary_0.xlsx
    """

    metro_manila_postcodes = tuple(x for x in range(400, 1849))
    luzon_province_postcodes = (
        tuple(x for x in range(1850, 5000))
        + tuple(x for x in range(5100, 5600))
    )
    visayas_province_postcodes = (
        tuple(x for x in range(5000, 5100))
        + tuple(x for x in range(5600, 5800))
        + tuple(x for x in range(6000, 6900))
    )
    mindanao_province_postcodes = (
        tuple(x for x in range(7000, 7600))
        + tuple(x for x in range(8000, 8900))
        + tuple(x for x in range(9000, 9900))
    )
    postcodes = (
        metro_manila_postcodes
        + luzon_province_postcodes
        + visayas_province_postcodes
        + mindanao_province_postcodes
    )
    metro_manila_lgus = (
        'Caloocan', 'Las Piñas', 'Makati', 'Malabon', 'Mandaluyong', 'Manila', 'Marikina', 'Muntinlupa', 'Navotas',
        'Parañaque', 'Pasay', 'Pasig', 'Pateros', 'Quezon City', 'San Juan', 'Taguig', 'Valenzuela',
    )
    province_lgus = (
        'Aborlan', 'Abra de Ilog', 'Abucay', 'Abulug', 'Abuyog', 'Adams', 'Agdangan', 'Aglipay', 'Agno', 'Agoncillo',
        'Agoo', 'Aguilar', 'Aguinaldo', 'Agutaya', 'Ajuy', 'Akbar', 'Al-Barka', 'Alabat', 'Alabel', 'Alamada',
        'Alaminos', 'Alangalang', 'Albuera', 'Alburquerque', 'Alcala', 'Alcantara', 'Alcoy', 'Alegria', 'Aleosan',
        'Alfonso Castañeda', 'Alfonso Lista', 'Alfonso', 'Aliaga', 'Alicia', 'Alilem', 'Alimodian', 'Alitagtag',
        'Allacapan', 'Allen', 'Almagro', 'Almeria', 'Aloguinsan', 'Aloran', 'Altavas', 'Alubijid', 'Amadeo',
        'Amai Manabilang', 'Ambaguio', 'Amlan', 'Ampatuan', 'Amulung', 'Anahawan', 'Anao', 'Anda', 'Angadanan', 'Angat',
        'Angeles', 'Angono', 'Anilao', 'Anini-y', 'Antequera', 'Antipas', 'Antipolo', 'Apalit', 'Aparri', 'Araceli',
        'Arakan', 'Arayat', 'Argao', 'Aringay', 'Aritao', 'Aroroy', 'Arteche', 'Asingan', 'Asipulo', 'Asturias',
        'Asuncion', 'Atimonan', 'Atok', 'Aurora', 'Ayungon', 'Baao', 'Babatngon', 'Bacacay', 'Bacarra', 'Baclayon',
        'Bacnotan', 'Baco', 'Bacolod-Kalawi', 'Bacolod', 'Bacolor', 'Bacong', 'Bacoor', 'Bacuag', 'Badian', 'Badiangan',
        'Badoc', 'Bagabag', 'Bagac', 'Bagamanoc', 'Baganga', 'Baggao', 'Bago', 'Baguio', 'Bagulin', 'Bagumbayan',
        'Bais', 'Bakun', 'Balabac', 'Balabagan', 'Balagtas', 'Balamban', 'Balanga', 'Balangiga', 'Balangkayan',
        'Balaoan', 'Balasan', 'Balatan', 'Balayan', 'Balbalan', 'Baleno', 'Baler', 'Balete', 'Baliangao', 'Baliguian',
        'Balilihan', 'Balindong', 'Balingasag', 'Balingoan', 'Baliuag', 'Ballesteros', 'Baloi', 'Balud', 'Balungao',
        'Bamban', 'Bambang', 'Banate', 'Banaue', 'Banaybanay', 'Banayoyo', 'Banga', 'Bangar', 'Bangued', 'Bangui',
        'Banguingui', 'Bani', 'Banisilan', 'Banna', 'Bansalan', 'Bansud', 'Bantay', 'Bantayan', 'Banton', 'Baras',
        'Barbaza', 'Barcelona', 'Barili', 'Barira', 'Barlig', 'Barobo', 'Barotac Nuevo', 'Barotac Viejo', 'Baroy',
        'Barugo', 'Basay', 'Basco', 'Basey', 'Basilisa', 'Basista', 'Basud', 'Batac', 'Batad', 'Batan', 'Batangas City',
        'Bataraza', 'Bato', 'Batuan', 'Bauan', 'Bauang', 'Bauko', 'Baungon', 'Bautista', 'Bay', 'Bayabas', 'Bayambang',
        'Bayang', 'Bayawan', 'Baybay', 'Bayog', 'Bayombong', 'Bayugan', 'Belison', 'Benito Soliven', 'Besao',
        'Bien Unido', 'Bilar', 'Biliran', 'Binalbagan', 'Binalonan', 'Biñan', 'Binangonan', 'Bindoy', 'Bingawan',
        'Binidayan', 'Binmaley', 'Binuangan', 'Biri', 'Bislig', 'Boac', 'Bobon', 'Bocaue', 'Bogo', 'Bokod', 'Bolinao',
        'Boliney', 'Boljoon', 'Bombon', 'Bongabon', 'Bongabong', 'Bongao', 'Bonifacio', 'Bontoc', 'Borbon', 'Borongan',
        'Boston', 'Botolan', 'Braulio E. Dujali', "Brooke's Point", 'Buadiposo-Buntong', 'Bubong', 'Bucay', 'Bucloc',
        'Buenavista', 'Bugallon', 'Bugasong', 'Buguey', 'Buguias', 'Buhi', 'Bula', 'Bulakan', 'Bulalacao', 'Bulan',
        'Buldon', 'Buluan', 'Bulusan', 'Bunawan', 'Burauen', 'Burdeos', 'Burgos', 'Buruanga', 'Bustos', 'Busuanga',
        'Butig', 'Butuan', 'Buug', 'Caba', 'Cabadbaran', 'Cabagan', 'Cabanatuan', 'Cabangan', 'Cabanglasan',
        'Cabarroguis', 'Cabatuan', 'Cabiao', 'Cabucgayan', 'Cabugao', 'Cabusao', 'Cabuyao', 'Cadiz', 'Cagayan de Oro',
        'Cagayancillo', 'Cagdianao', 'Cagwait', 'Caibiran', 'Cainta', 'Cajidiocan', 'Calabanga', 'Calaca', 'Calamba',
        'Calanasan', 'Calanogas', 'Calapan', 'Calape', 'Calasiao', 'Calatagan', 'Calatrava', 'Calauag', 'Calauan',
        'Calayan', 'Calbayog', 'Calbiga', 'Calinog', 'Calintaan', 'Calubian', 'Calumpit', 'Caluya', 'Camalaniugan',
        'Camalig', 'Camaligan', 'Camiling', 'Can-avid', 'Canaman', 'Candaba', 'Candelaria', 'Candijay', 'Candon',
        'Candoni', 'Canlaon', 'Cantilan', 'Caoayan', 'Capalonga', 'Capas', 'Capoocan', 'Capul', 'Caraga', 'Caramoan',
        'Caramoran', 'Carasi', 'Carcar', 'Cardona', 'Carigara', 'Carles', 'Carmen', 'Carmona', 'Carranglan',
        'Carrascal', 'Casiguran', 'Castilla', 'Castillejos', 'Cataingan', 'Catanauan', 'Catarman', 'Catbalogan',
        'Cateel', 'Catigbian', 'Catmon', 'Catubig', 'Cauayan', 'Cavinti', 'Cavite City', 'Cawayan', 'Cebu City',
        'Cervantes', 'Clarin', 'Claver', 'Claveria', 'Columbio', 'Compostela', 'Concepcion', 'Conner', 'Consolacion',
        'Corcuera', 'Cordon', 'Cordova', 'Corella', 'Coron', 'Cortes', 'Cotabato City', 'Cuartero', 'Cuenca', 'Culaba',
        'Culasi', 'Culion', 'Currimao', 'Cuyapo', 'Cuyo', 'Daanbantayan', 'Daet', 'Dagami', 'Dagohoy', 'Daguioman',
        'Dagupan', 'Dalaguete', 'Damulog', 'Danao', 'Dangcagan', 'Danglas', 'Dao', 'Dapa', 'Dapitan', 'Daraga', 'Daram',
        'Dasmariñas', 'Dasol', 'Datu Abdullah Sangki', 'Datu Anggal Midtimbang', 'Datu Blah T. Sinsuat',
        'Datu Hoffer Ampatuan', 'Datu Montawal', 'Datu Odin Sinsuat', 'Datu Paglas', 'Datu Piang', 'Datu Salibo',
        'Datu Saudi-Ampatuan', 'Datu Unsay', 'Dauin', 'Dauis', 'Davao City', 'Del Carmen', 'Del Gallego',
        'Delfin Albano', 'Diadi', 'Diffun', 'Digos', 'Dilasag', 'Dimasalang', 'Dimataling', 'Dimiao', 'Dinagat',
        'Dinalungan', 'Dinalupihan', 'Dinapigue', 'Dinas', 'Dingalan', 'Dingle', 'Dingras', 'Dipaculao', 'Diplahan',
        'Dipolog', 'Ditsaan-Ramain', 'Divilacan', 'Dolores', 'Don Carlos', 'Don Marcelino', 'Don Victoriano Chiongbian',
        'Doña Remedios Trinidad', 'Donsol', 'Dueñas', 'Duero', 'Dulag', 'Dumaguete', 'Dumalag', 'Dumalinao', 'Dumalneg',
        'Dumangas', 'Dumanjug', 'Dumaran', 'Dumarao', 'Dumingag', 'Dupax del Norte', 'Dupax del Sur', 'Echague',
        'El Nido', 'El Salvador', 'Enrile', 'Enrique B. Magalona', 'Enrique Villanueva', 'Escalante', 'Esperanza',
        'Estancia', 'Famy', 'Ferrol', 'Flora', 'Floridablanca', 'Gabaldon', 'Gainza', 'Galimuyod', 'Gamay', 'Gamu',
        'Ganassi', 'Gandara', 'Gapan', 'Garchitorena', 'Garcia Hernandez', 'Gasan', 'Gattaran',
        'General Emilio Aguinaldo', 'General Luna', 'General MacArthur', 'General Mamerto Natividad',
        'General Mariano Alvarez', 'General Nakar', 'General Salipada K. Pendatun', 'General Santos', 'General Tinio',
        'General Trias', 'Gerona', 'Getafe', 'Gigaquit', 'Gigmoto', 'Ginatilan', 'Gingoog', 'Giporlos', 'Gitagum',
        'Glan', 'Gloria', 'Goa', 'Godod', 'Gonzaga', 'Governor Generoso', 'Gregorio del Pilar', 'Guagua', 'Gubat',
        'Guiguinto', 'Guihulngan', 'Guimba', 'Guimbal', 'Guinayangan', 'Guindulman', 'Guindulungan', 'Guinobatan',
        'Guinsiliban', 'Guipos', 'Guiuan', 'Gumaca', 'Gutalac', 'Hadji Mohammad Ajul', 'Hadji Muhtamad',
        'Hadji Panglima Tahil', 'Hagonoy', 'Hamtic', 'Hermosa', 'Hernani', 'Hilongos', 'Himamaylan', 'Hinabangan',
        'Hinatuan', 'Hindang', 'Hingyon', 'Hinigaran', 'Hinoba-an', 'Hinunangan', 'Hinundayan', 'Hungduan', 'Iba',
        'Ibaan', 'Ibajay', 'Igbaras', 'Iguig', 'Ilagan', 'Iligan', 'Ilog', 'Iloilo City', 'Imelda', 'Impasugong',
        'Imus', 'Inabanga', 'Indanan', 'Indang', 'Infanta', 'Initao', 'Inopacan', 'Ipil', 'Iriga', 'Irosin', 'Isabel',
        'Isabela City', 'Isabela', 'Isulan', 'Itbayat', 'Itogon', 'Ivana', 'Ivisan', 'Jabonga', 'Jaen', 'Jagna',
        'Jalajala', 'Jamindan', 'Janiuay', 'Jaro', 'Jasaan', 'Javier', 'Jiabong', 'Jimalalud', 'Jimenez', 'Jipapad',
        'Jolo', 'Jomalig', 'Jones', 'Jordan', 'Jose Abad Santos', 'Jose Dalman', 'Jose Panganiban', 'Josefina',
        'Jovellar', 'Juban', 'Julita', 'Kabacan', 'Kabankalan', 'Kabasalan', 'Kabayan', 'Kabugao', 'Kabuntalan',
        'Kadingilan', 'Kalamansig', 'Kalawit', 'Kalayaan', 'Kalibo', 'Kalilangan', 'Kalingalan Caluang', 'Kananga',
        'Kapai', 'Kapalong', 'Kapangan', 'Kapatagan', 'Kasibu', 'Katipunan', 'Kauswagan', 'Kawayan', 'Kawit', 'Kayapa',
        'Kiamba', 'Kiangan', 'Kibawe', 'Kiblawan', 'Kibungan', 'Kidapawan', 'Kinoguitan', 'Kitaotao', 'Kitcharao',
        'Kolambugan', 'Koronadal', 'Kumalarang', 'La Carlota', 'La Castellana', 'La Libertad', 'La Paz', 'La Trinidad',
        'Laak', 'Labangan', 'Labason', 'Labo', 'Labrador', 'Lacub', 'Lagangilang', 'Lagawe', 'Lagayan', 'Lagonglong',
        'Lagonoy', 'Laguindingan', 'Lake Sebu', 'Lakewood', 'Lal-lo', 'Lala', 'Lambayong', 'Lambunao', 'Lamitan',
        'Lamut', 'Langiden', 'Languyan', 'Lantapan', 'Lantawan', 'Lanuza', 'Laoac', 'Laoag', 'Laoang', 'Lapinig',
        'Lapu-Lapu', 'Lapuyan', 'Larena', 'Las Navas', 'Las Nieves', 'Lasam', 'Laua-an', 'Laur', 'Laurel', 'Lavezares',
        'Lawaan', 'Lazi', 'Lebak', 'Leganes', 'Legazpi', 'Lemery', 'Leon B. Postigo', 'Leon', 'Leyte', 'Lezo', 'Lian',
        'Lianga', 'Libacao', 'Libagon', 'Libertad', 'Libjo', 'Libmanan', 'Libon', 'Libona', 'Libungan', 'Licab',
        'Licuan-Baay', 'Lidlidda', 'Ligao', 'Lila', 'Liliw', 'Liloan', 'Liloy', 'Limasawa', 'Limay', 'Linamon',
        'Linapacan', 'Lingayen', 'Lingig', 'Lipa', 'Llanera', 'Llorente', 'Loay', 'Lobo', 'Loboc', 'Looc', 'Loon',
        'Lope de Vega', 'Lopez Jaena', 'Lopez', 'Loreto', 'Los Baños', 'Luba', 'Lubang', 'Lubao', 'Lubuagan', 'Lucban',
        'Lucena', 'Lugait', 'Lugus', 'Luisiana', 'Lumba-Bayabao', 'Lumbaca-Unayan', 'Lumban', 'Lumbatan',
        'Lumbayanague', 'Luna', 'Lupao', 'Lupi', 'Lupon', 'Lutayan', 'Luuk', "M'lang", 'Maasim', 'Maasin', 'Maayon',
        'Mabalacat', 'Mabinay', 'Mabini', 'Mabitac', 'Mabuhay', 'Macabebe', 'Macalelon', 'MacArthur', 'Maco',
        'Maconacon', 'Macrohon', 'Madalag', 'Madalum', 'Madamba', 'Maddela', 'Madrid', 'Madridejos', 'Magalang',
        'Magallanes', 'Magarao', 'Magdalena', 'Magdiwang', 'Magpet', 'Magsaysay', 'Magsingal', 'Maguing', 'Mahaplag',
        'Mahatao', 'Mahayag', 'Mahinog', 'Maigo', 'Maimbung', 'Mainit', 'Maitum', 'Majayjay', 'Makato', 'Makilala',
        'Malabang', 'Malabuyoc', 'Malalag', 'Malangas', 'Malapatan', 'Malasiqui', 'Malay', 'Malaybalay', 'Malibcong',
        'Malilipot', 'Malimono', 'Malinao', 'Malita', 'Malitbog', 'Mallig', 'Malolos', 'Malungon', 'Maluso', 'Malvar',
        'Mamasapano', 'Mambajao', 'Mamburao', 'Mambusao', 'Manabo', 'Manaoag', 'Manapla', 'Manay', 'Mandaon', 'Mandaue',
        'Mangaldan', 'Mangatarem', 'Mangudadatu', 'Manito', 'Manjuyod', 'Mankayan', 'Manolo Fortich', 'Mansalay',
        'Manticao', 'Manukan', 'Mapanas', 'Mapandan', 'Mapun', 'Marabut', 'Maragondon', 'Maragusan', 'Maramag',
        'Marantao', 'Marawi', 'Marcos', 'Margosatubig', 'Maria Aurora', 'Maria', 'Maribojoc', 'Marihatag', 'Marilao',
        'Maripipi', 'Mariveles', 'Marogong', 'Masantol', 'Masbate City', 'Masinloc', 'Masiu', 'Maslog', 'Mataasnakahoy',
        'Matag-ob', 'Matalam', 'Matalom', 'Matanao', 'Matanog', 'Mati', 'Matnog', 'Matuguinao', 'Matungao', 'Mauban',
        'Mawab', 'Mayantoc', 'Maydolong', 'Mayorga', 'Mayoyao', 'Medellin', 'Medina', 'Mendez', 'Mercedes', 'Merida',
        'Mexico', 'Meycauayan', 'Miagao', 'Midsalip', 'Midsayap', 'Milagros', 'Milaor', 'Mina', 'Minalabac', 'Minalin',
        'Minglanilla', 'Moalboal', 'Mobo', 'Mogpog', 'Moises Padilla', 'Molave', 'Moncada', 'Mondragon', 'Monkayo',
        'Monreal', 'Montevista', 'Morong', 'Motiong', 'Mulanay', 'Mulondo', 'Munai', 'Muñoz', 'Murcia', 'Mutia',
        'Naawan', 'Nabas', 'Nabua', 'Nabunturan', 'Naga', 'Nagbukel', 'Nagcarlan', 'Nagtipunan', 'Naguilian', 'Naic',
        'Nampicuan', 'Narra', 'Narvacan', 'Nasipit', 'Nasugbu', 'Natividad', 'Natonin', 'Naujan', 'Naval', 'New Bataan',
        'New Corella', 'New Lucena', 'New Washington', 'Norala', 'Northern Kabuntalan', 'Norzagaray', 'Noveleta',
        'Nueva Era', 'Nueva Valencia', 'Numancia', 'Nunungan', 'Oas', 'Obando', 'Ocampo', 'Odiongan', 'Old Panamao',
        'Olongapo', 'Olutanga', 'Omar', 'Opol', 'Orani', 'Oras', 'Orion', 'Ormoc', 'Oroquieta', 'Oslob', 'Oton',
        'Ozamiz', 'Padada', 'Padre Burgos', 'Padre Garcia', 'Paete', 'Pagadian', 'Pagalungan', 'Pagayawan', 'Pagbilao',
        'Paglat', 'Pagsanghan', 'Pagsanjan', 'Pagudpud', 'Pakil', 'Palanan', 'Palanas', 'Palapag', 'Palauig', 'Palayan',
        'Palimbang', 'Palo', 'Palompon', 'Paluan', 'Pambujan', 'Pamplona', 'Panabo', 'Panaon', 'Panay', 'Pandag',
        'Pandami', 'Pandan', 'Pandi', 'Panganiban', 'Pangantucan', 'Pangil', 'Panglao', 'Panglima Estino',
        'Panglima Sugala', 'Pangutaran', 'Paniqui', 'Panitan', 'Pantabangan', 'Pantao Ragat', 'Pantar', 'Pantukan',
        'Panukulan', 'Paoay', 'Paombong', 'Paracale', 'Paracelis', 'Paranas', 'Parang', 'Pasacao', 'Pasil', 'Passi',
        'Pastrana', 'Pasuquin', 'Pata', 'Patikul', 'Patnanungan', 'Patnongon', 'Pavia', 'Payao', 'Peñablanca',
        'Peñaranda', 'Peñarrubia', 'Perez', 'Piagapo', 'Piat', 'Picong', 'Piddig', 'Pidigan', 'Pigcawayan', 'Pikit',
        'Pila', 'Pilar', 'Pili', 'Pililla', 'Pinabacdao', 'Pinamalayan', 'Pinamungajan', 'Piñan', 'Pinili', 'Pintuyan',
        'Pinukpuk', 'Pio Duran', 'Pio V. Corpuz', 'Pitogo', 'Placer', 'Plaridel', 'Pola', 'Polanco', 'Polangui',
        'Polillo', 'Polomolok', 'Pontevedra', 'Poona Bayabao', 'Poona Piagapo', 'Porac', 'Poro', 'Pototan',
        'Pozorrubio', 'Presentacion', 'President Carlos P. Garcia', 'President Manuel A. Roxas', 'President Quirino',
        'President Roxas', 'Prieto Diaz', 'Prosperidad', 'Pualas', 'Pudtol', 'Puerto Galera', 'Puerto Princesa', 'Pugo',
        'Pulilan', 'Pulupandan', 'Pura', 'Quezon', 'Quinapondan', 'Quirino', 'Ragay', 'Rajah Buayan', 'Ramon Magsaysay',
        'Ramon', 'Ramos', 'Rapu-Rapu', 'Real', 'Reina Mercedes', 'Remedios T. Romualdez', 'Rizal', 'Rodriguez',
        'Romblon', 'Ronda', 'Rosales', 'Rosario', 'Roseller Lim', 'Roxas City', 'Roxas', 'Sabangan', 'Sablan',
        'Sablayan', 'Sabtang', 'Sadanga', 'Sagada', 'Sagay', 'Sagbayan', 'Sagñay', 'Saguday', 'Saguiaran',
        'Saint Bernard', 'Salay', 'Salcedo', 'Sallapadan', 'Salug', 'Salvador Benedicto', 'Salvador', 'Samal',
        'Samboan', 'Sampaloc', 'San Agustin', 'San Andres', 'San Antonio', 'San Benito', 'San Carlos', 'San Clemente',
        'San Dionisio', 'San Emilio', 'San Enrique', 'San Esteban', 'San Fabian', 'San Felipe', 'San Fernando',
        'San Francisco', 'San Gabriel', 'San Guillermo', 'San Ildefonso', 'San Isidro', 'San Jacinto', 'San Joaquin',
        'San Jorge', 'San Jose de Buan', 'San Jose de Buenavista', 'San Jose del Monte', 'San Jose', 'San Juan',
        'San Julian', 'San Leonardo', 'San Lorenzo Ruiz', 'San Lorenzo', 'San Luis', 'San Manuel', 'San Marcelino',
        'San Mariano', 'San Mateo', 'San Miguel', 'San Narciso', 'San Nicolas', 'San Pablo', 'San Pascual', 'San Pedro',
        'San Policarpo', 'San Quintin', 'San Rafael', 'San Remigio', 'San Ricardo', 'San Roque', 'San Sebastian',
        'San Simon', 'San Teodoro', 'San Vicente', 'Sanchez-Mira', 'Santa Ana', 'Santa Barbara', 'Santa Catalina',
        'Santa Cruz', 'Santa Elena', 'Santa Fe', 'Santa Ignacia', 'Santa Josefa', 'Santa Lucia', 'Santa Magdalena',
        'Santa Marcela', 'Santa Margarita', 'Santa Maria', 'Santa Monica', 'Santa Praxedes', 'Santa Rita', 'Santa Rosa',
        'Santa Teresita', 'Santa', 'Santander', 'Santiago', 'Santo Domingo', 'Santo Niño', 'Santo Tomas', 'Santol',
        'Sapa-Sapa', 'Sapad', 'Sapang Dalaga', 'Sapian', 'Sara', 'Sarangani', 'Sariaya', 'Sarrat', 'Sasmuan', 'Sebaste',
        'Senator Ninoy Aquino', 'Sergio Osmeña Sr.', 'Sevilla', 'Shariff Aguak', 'Shariff Saydona Mustapha', 'Siasi',
        'Siaton', 'Siay', 'Siayan', 'Sibagat', 'Sibalom', 'Sibonga', 'Sibuco', 'Sibulan', 'Sibunag', 'Sibutad',
        'Sibutu', 'Sierra Bullones', 'Sigay', 'Sigma', 'Sikatuna', 'Silago', 'Silang', 'Silay', 'Silvino Lobos',
        'Simunul', 'Sinacaban', 'Sinait', 'Sindangan', 'Siniloan', 'Siocon', 'Sipalay', 'Sipocot', 'Siquijor',
        'Sirawai', 'Siruma', 'Sison', 'Sitangkai', 'Socorro', 'Sofronio Española', 'Sogod', 'Solana', 'Solano',
        'Solsona', 'Sominot', 'Sorsogon City', 'South Ubian', 'South Upi', 'Sual', 'Subic', 'Sudipen', 'Sugbongcogon',
        'Sugpon', 'Sulat', 'Sulop', 'Sultan Dumalondong', 'Sultan Kudarat', 'Sultan Mastura', 'Sultan Naga Dimaporo',
        'Sultan sa Barongis', 'Sultan Sumagka', 'Sumilao', 'Sumisip', 'Surallah', 'Surigao City', 'Suyo', "T'Boli",
        'Taal', 'Tabaco', 'Tabango', 'Tabina', 'Tabogon', 'Tabontabon', 'Tabuan-Lasa', 'Tabuelan', 'Tabuk', 'Tacloban',
        'Tacurong', 'Tadian', 'Taft', 'Tagana-an', 'Tagapul-an', 'Tagaytay', 'Tagbilaran', 'Tagbina', 'Tagkawayan',
        'Tago', 'Tagoloan II', 'Tagoloan', 'Tagudin', 'Tagum', 'Talacogon', 'Talaingod', 'Talakag', 'Talalora',
        'Talavera', 'Talayan', 'Talibon', 'Talipao', 'Talisay', 'Talisayan', 'Talugtug', 'Talusan', 'Tambulig',
        'Tampakan', 'Tamparan', 'Tampilisan', 'Tanauan', 'Tanay', 'Tandag', 'Tandubas', 'Tangalan', 'Tangcal', 'Tangub',
        'Tanjay', 'Tantangan', 'Tanudan', 'Tanza', 'Tapaz', 'Tapul', 'Taraka', 'Tarangnan', 'Tarlac City', 'Tarragona',
        'Tayabas', 'Tayasan', 'Taysan', 'Taytay', 'Tayug', 'Tayum', 'Teresa', 'Ternate', 'Tiaong', 'Tibiao', 'Tigaon',
        'Tigbao', 'Tigbauan', 'Tinambac', 'Tineg', 'Tinglayan', 'Tingloy', 'Tinoc', 'Tipo-Tipo', 'Titay', 'Tiwi',
        'Tobias Fornier', 'Toboso', 'Toledo', 'Tolosa', 'Tomas Oppus', 'Torrijos', 'Trece Martires', 'Trento',
        'Trinidad', 'Tuao', 'Tuba', 'Tubajon', 'Tubao', 'Tubaran', 'Tubay', 'Tubigon', 'Tublay', 'Tubo', 'Tubod',
        'Tubungan', 'Tuburan', 'Tudela', 'Tugaya', 'Tuguegarao', 'Tukuran', 'Tulunan', 'Tumauini', 'Tunga', 'Tungawan',
        'Tupi', 'Turtle Islands', 'Tuy', 'Ubay', 'Umingan', 'Ungkaya Pukan', 'Unisan', 'Upi', 'Urbiztondo', 'Urdaneta',
        'Uson', 'Uyugan', 'Valderrama', 'Valencia', 'Valladolid', 'Vallehermoso', 'Veruela', 'Victoria', 'Victorias',
        'Viga', 'Vigan', 'Villaba', 'Villanueva', 'Villareal', 'Villasis', 'Villaverde', 'Villaviciosa',
        'Vincenzo A. Sagun', 'Vintar', 'Vinzons', 'Virac', 'Wao', 'Zamboanga City', 'Zamboanguita', 'Zaragoza',
        'Zarraga', 'Zumarraga',
    )
    luzon_provinces = (
        'Abra', 'Albay', 'Apayao', 'Aurora', 'Bataan', 'Batanes', 'Batangas', 'Benguet', 'Bulacan', 'Cagayan',
        'Camarines Norte', 'Camarines Sur', 'Catanduanes', 'Cavite', 'Ifugao', 'Ilocos Norte', 'Ilocos Sur', 'Isabela',
        'Kalinga', 'La Union', 'Laguna', 'Marinduque', 'Masbate', 'Mountain Province', 'Nueva Ecija', 'Nueva Vizcaya',
        'Occidental Mindoro', 'Oriental Mindoro', 'Palawan', 'Pampanga', 'Pangasinan', 'Quezon', 'Quirino', 'Rizal',
        'Romblon', 'Sorsogon', 'Tarlac', 'Zambales',
    )
    visayas_provinces = (
        'Aklan', 'Antique', 'Biliran', 'Bohol', 'Capiz', 'Cebu', 'Eastern Samar', 'Guimaras', 'Iloilo', 'Leyte',
        'Negros Occidental', 'Negros Oriental', 'Northern Samar', 'Samar', 'Siquijor', 'Southern Leyte',
    )
    mindanao_provinces = (
        'Agusan del Norte', 'Agusan del Sur', 'Basilan', 'Bukidnon', 'Camiguin', 'Compostela Valley', 'Cotabato',
        'Davao del Norte', 'Davao del Sur', 'Davao Occidental', 'Davao Oriental', 'Dinagat Islands', 'Lanao del Norte',
        'Lanao del Sur', 'Maguindanao', 'Misamis Occidental', 'Misamis Oriental', 'Sarangani', 'South Cotabato',
        'Sultan Kudarat', 'Sulu', 'Surigao del Norte', 'Surigao del Sur', 'Tawi-Tawi', 'Zamboanga del Norte',
        'Zamboanga del Sur', 'Zamboanga Sibugay',
    )
    provinces = luzon_provinces + visayas_provinces + mindanao_provinces

    partitioned_building_number_formats = (
        '{{standalone_building_number}}?',
        '{{standalone_building_number}} ?',
        '{{standalone_building_number}}-?',
        '{{standalone_building_number}} Unit ?',
    )
    building_unit_number_formats = (
        'Unit {{floor_unit_number}}',
        'Room {{floor_unit_number}}',
        '{{floor_number}}F',
        '{{ordinal_floor_number}} Floor',
    )
    building_name_formats = (
        '{{last_name}} {{building_name_suffix}}',
        '{{random_object_name}} {{building_name_suffix}}',
    )
    building_name_suffixes = (
        'Apartment', 'Apartments',
        'Building', 'Building %', 'Building Tower %',
        'Condominiums', 'Condominiums %', 'Condominiums Tower %',
        'Place', 'Place %', 'Place Tower %',
        'Residences', 'Residences %', 'Residences Tower %',
        'Suites', 'Suites %', 'Suites Tower %',
        'Tower', 'Towers', 'Towers %',
    )
    subdivision_unit_number_formats = (
        'B{{subdivision_block_number}} L{{subdivision_lot_number}}',
        'Block {{subdivision_block_number}} Lot {{subdivision_lot_number}}',
    )
    subdivision_name_formats = (
        '{{last_name}} {{subdivision_name_suffix}}',
        '{{random_object_name}} {{subdivision_name_suffix}}',
    )
    subdivision_name_suffixes = (
        'Cove', 'Cove %', 'Cove Phase %',
        'Estates', 'Estates %', 'Estates Phase %',
        'Grove', 'Grove %', 'Grove Phase %',
        'Homes', 'Homes %', 'Homes Phase %',
        'Subdivision', 'Subdivision %', 'Subdivision Phase %',
        'Village', 'Village %', 'Village Phase %',
    )
    floor_numbers = OrderedDict([
        (str(x), 0.08) for x in range(2, 5)  # Floors 2 to 4, 24% of the time
    ] + [
        (str(x), 0.32356832089420257 / x) for x in range(5, 13)  # Floors 5 to 12, 33% of the time
    ] + [
        (str(x), 0.30341265418486174 / (x - 1)) for x in range(14, 30)  # Floors 14 to 29, 25% of the time
    ] + [
        (str(x), 0.30096338222652870 / (x - 1)) for x in range(30, 50)  # Floors 30 to 49, 16% of the time
    ] + [
        (str(x), 0.04570476167856688 / (x - 1)) for x in range(50, 75)  # Floors 50 to 74, 1.9% of the time
    ] + [
        (str(x), 0.003415677066138734 / (x - 1)) for x in range(75, 100)  # Floors 75 to 99, 0.1% of the time
    ])

    street_suffixes = OrderedDict([
        ('Avenue', 0.12),
        ('Avenue Extension', 0.01),
        ('Boulevard', 0.05),
        ('Boulevard Extension', 0.008),
        ('Circle', 0.002),
        ('Drive', 0.15),
        ('Drive Extension', 0.03),
        ('Expressway', 0.01),
        ('Extension', 0.05),
        ('Highway', 0.02),
        ('Road', 0.2),
        ('Road Extension', 0.04),
        ('Service Road', 0.01),
        ('Street', 0.3),
    ])
    street_name_formats = (
        '{{last_name}} {{street_suffix}}',
        '{{ordinal_street_number}} {{street_suffix}}',
        '{{gemstone_name}} {{street_suffix}}',
        '{{mountain_name}} {{street_suffix}}',
        '{{plant_name}} {{street_suffix}}',
        '{{space_object_name}} {{street_suffix}}',
    )
    street_address_formats = (
        '{{standalone_building_number}} {{street_name}}',
        '{{partitioned_building_number}} {{street_name}}',
        '{{subdivision_unit_number}} {{subdivision_name}}, {{street_name}}',
        '{{subdivision_unit_number}} {{street_name}}, {{subdivision_name}}',
        '{{standalone_building_number}} {{street_name}}, {{subdivision_name}}',
        '{{building_unit_number}} {{building_name}}, {{standalone_building_number}} {{street_name}}',
    )

    metro_manila_address_formats = (
        '{{street_address}}, {{metro_manila_lgu}}, {{metro_manila_postcode}} Metro Manila',
    )
    luzon_province_address_formats = (
        '{{street_address}}, {{province_lgu}}, {{luzon_province_postcode}} {{luzon_province}}',
    )
    visayas_province_address_formats = (
        '{{street_address}}, {{province_lgu}}, {{visayas_province_postcode}} {{visayas_province}}',
    )
    mindanao_province_address_formats = (
        '{{street_address}}, {{province_lgu}}, {{mindanao_province_postcode}} {{mindanao_province}}',
    )
    address_formats = OrderedDict([
        (metro_manila_address_formats, 0.127524),
        (luzon_province_address_formats, 0.485317),
        (visayas_province_address_formats, 0.148142),
        (mindanao_province_address_formats, 0.239017),
    ])

    def _ordinal_string(self, num):
        if isinstance(num, str):
            num = int(num)
        suffix = ['th', 'st', 'nd', 'rd', 'th'][min(num % 10, 4)]
        if 11 <= num % 100 <= 13:
            suffix = 'th'
        return str(num) + suffix

    def _create_postcode(self, postcodes):
        return f'{self.random_element(postcodes):04d}'

    def _create_address(self, address_formats):
        return self.generator.parse(self.random_element(address_formats))

    def metro_manila_postcode(self):
        return self._create_postcode(self.metro_manila_postcodes)

    def luzon_province_postcode(self):
        return self._create_postcode(self.luzon_province_postcodes)

    def visayas_province_postcode(self):
        return self._create_postcode(self.visayas_province_postcodes)

    def mindanao_province_postcode(self):
        return self._create_postcode(self.mindanao_province_postcodes)

    def postcode(self):
        return self._create_postcode(self.postcodes)

    def luzon_province(self):
        return self.random_element(self.luzon_provinces)

    def visayas_province(self):
        return self.random_element(self.visayas_provinces)

    def mindanao_province(self):
        return self.random_element(self.mindanao_provinces)

    def province(self):
        return self.random_element(self.provinces)

    def standalone_building_number(self):
        return str(self.random_int(min=1))

    def partitioned_building_number(self):
        pattern = self.lexify(
            self.random_element(self.partitioned_building_number_formats), letters=ascii_uppercase[:10],
        )
        return self.generator.parse(pattern)

    def building_number(self):
        if self.random_int() % 2 == 0:
            return self.standalone_building_number()
        else:
            return self.partitioned_building_number()

    def ordinal_street_number(self):
        return self._ordinal_string(self.random_int(1, 99))

    def floor_number(self):
        return self.random_element(self.floor_numbers)

    def ordinal_floor_number(self):
        return self._ordinal_string(self.floor_number())

    def floor_unit_number(self):
        return f'{self.floor_number()}{self.random_int(1, 40):02d}'

    def building_unit_number(self):
        return self.generator.parse(self.random_element(self.building_unit_number_formats))

    def building_name(self):
        return self.generator.parse(self.random_element(self.building_name_formats))

    def building_name_suffix(self):
        return self.numerify(self.random_element(self.building_name_suffixes))

    def subdivision_block_number(self):
        return f'{self.random_int(1, 25):02d}'

    def subdivision_lot_number(self):
        return f'{self.random_int(1, 99):02d}'

    def subdivision_unit_number(self):
        return self.generator.parse(self.random_element(self.subdivision_unit_number_formats))

    def subdivision_name(self):
        return self.generator.parse(self.random_element(self.subdivision_name_formats))

    def subdivision_name_suffix(self):
        return self.numerify(self.random_element(self.subdivision_name_suffixes))

    def metro_manila_lgu(self):
        return self.random_element(self.metro_manila_lgus)

    def province_lgu(self):
        return self.random_element(self.province_lgus)

    def metro_manila_address(self):
        return self._create_address(self.metro_manila_address_formats)

    def luzon_province_address(self):
        return self._create_address(self.luzon_province_address_formats)

    def visayas_province_address(self):
        return self._create_address(self.visayas_province_address_formats)

    def mindanao_province_address(self):
        return self._create_address(self.mindanao_province_address_formats)

    def address(self):
        return self._create_address(self.random_element(self.address_formats))



from functools import wraps
from typing import Any, Callable, Dict, Tuple

from faker.utils import text


def slugify(fn: Callable) -> Callable:
    @wraps(fn)
    def wrapper(*args: Tuple[Any, ...], **kwargs: Dict[str, Any]) -> str:
        return text.slugify(fn(*args, **kwargs))
    return wrapper


def slugify_domain(fn: Callable) -> Callable:
    @wraps(fn)
    def wrapper(*args: Tuple[Any, ...], **kwargs: Dict[str, Any]) -> str:
        return text.slugify(fn(*args, **kwargs), allow_dots=True)
    return wrapper


def slugify_unicode(fn: Callable) -> Callable:
    @wraps(fn)
    def wrapper(*args: Tuple[Any, ...], **kwargs: Dict[str, Any]) -> str:
        return text.slugify(fn(*args, **kwargs), allow_unicode=True)
    return wrapper


def lowercase(fn: Callable) -> Callable:
    @wraps(fn)
    def wrapper(*args: Tuple[Any, ...], **kwargs: Dict[str, Any]) -> str:
        return fn(*args, **kwargs).lower()
    return wrapper



from faker.utils.checksums import calculate_luhn

from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        '{{last_name}} {{company_suffix}}',
        '{{last_name}}-{{last_name}} {{company_suffix}}',
        '{{last_name}}, {{last_name}} e {{last_name}} {{company_suffix}}',
    )

    catch_phrase_words = (
        ('Abilità',
         'Access',
         'Adattatore',
         'Algoritmo',
         'Alleanza',
         'Analizzatore',
         'Applicazione',
         'Approccio',
         'Architettura',
         'Archivio',
         'Intelligenza artificiale',
         'Array',
         'Attitudine',
         'Benchmark',
         'Capacità',
         'Sfida',
         'Circuito',
         'Collaborazione',
         'Complessità',
         'Concetto',
         'Conglomerato',
         'Contingenza',
         'Core',
         'Database',
         'Data-warehouse',
         'Definizione',
         'Emulazione',
         'Codifica',
         'Criptazione',
         'Firmware',
         'Flessibilità',
         'Previsione',
         'Frame',
         'framework',
         'Funzione',
         'Funzionalità',
         'Interfaccia grafica',
         'Hardware',
         'Help-desk',
         'Gerarchia',
         'Hub',
         'Implementazione',
         'Infrastruttura',
         'Iniziativa',
         'Installazione',
         'Set di istruzioni',
         'Interfaccia',
         'Soluzione internet',
         'Intranet',
         'Conoscenza base',
         'Matrici',
         'Matrice',
         'Metodologia',
         'Middleware',
         'Migrazione',
         'Modello',
         'Moderazione',
         'Monitoraggio',
         'Moratoria',
         'Rete',
         'Architettura aperta',
         'Sistema aperto',
         'Orchestrazione',
         'Paradigma',
         'Parallelismo',
         'Policy',
         'Portale',
         'Struttura di prezzo',
         'Prodotto',
         'Produttività',
         'Progetto',
         'Proiezione',
         'Protocollo',
         'Servizio clienti',
         'Software',
         'Soluzione',
         'Standardizzazione',
         'Strategia',
         'Struttura',
         'Successo',
         'Sovrastruttura',
         'Supporto',
         'Sinergia',
         'Task-force',
         'Finestra temporale',
         'Strumenti',
         'Utilizzazione',
         'Sito web',
         'Forza lavoro'),
        ('adattiva',
         'avanzata',
         'migliorata',
         'assimilata',
         'automatizzata',
         'bilanciata',
         'centralizzata',
         'compatibile',
         'configurabile',
         'cross-platform',
         'decentralizzata',
         'digitalizzata',
         'distribuita',
         'piccola',
         'ergonomica',
         'esclusiva',
         'espansa',
         'estesa',
         'configurabile',
         'fondamentale',
         'orizzontale',
         'implementata',
         'innovativa',
         'integrata',
         'intuitiva',
         'inversa',
         'gestita',
         'obbligatoria',
         'monitorata',
         'multi-canale',
         'multi-laterale',
         'open-source',
         'operativa',
         'ottimizzata',
         'organica',
         'persistente',
         'polarizzata',
         'proattiva',
         'programmabile',
         'progressiva',
         'reattiva',
         'riallineata',
         'ricontestualizzata',
         'ridotta',
         'robusta',
         'sicura',
         'condivisibile',
         'stand-alone',
         'switchabile',
         'sincronizzata',
         'sinergica',
         'totale',
         'universale',
         'user-friendly',
         'versatile',
         'virtuale',
         'visionaria'),
        ('24 ore',
         '24/7',
         'terza generazione',
         'quarta generazione',
         'quinta generazione',
         'sesta generazione',
         'asimmetrica',
         'asincrona',
         'background',
         'bi-direzionale',
         'biforcata',
         'bottom-line',
         'coerente',
         'coesiva',
         'composita',
         'sensibile al contesto',
         'basta sul contesto',
         'basata sul contenuto',
         'dedicata',
         'didattica',
         'direzionale',
         'discreta',
         'dinamica',
         'eco-centrica',
         'esecutiva',
         'esplicita',
         'full-range',
         'globale',
         'euristica',
         'alto livello',
         'olistica',
         'omogenea',
         'ibrida',
         'impattante',
         'incrementale',
         'intangibile',
         'interattiva',
         'intermediaria',
         'locale',
         'logistica',
         'massimizzata',
         'metodica',
         'mission-critical',
         'mobile',
         'modulare',
         'motivazionale',
         'multimedia',
         'multi-tasking',
         'nazionale',
         'neutrale',
         'nextgeneration',
         'non-volatile',
         'object-oriented',
         'ottima',
         'ottimizzante',
         'radicale',
         'real-time',
         'reciproca',
         'regionale',
         'responsiva',
         'scalabile',
         'secondaria',
         'stabile',
         'statica',
         'sistematica',
         'sistemica',
         'tangibile',
         'terziaria',
         'uniforme',
         'valore aggiunto'))

    bsWords = (
        ('partnerships',
         'comunità',
         'ROI',
         'soluzioni',
         'e-services',
         'nicchie',
         'tecnologie',
         'contenuti',
         'supply-chains',
         'convergenze',
         'relazioni',
         'architetture',
         'interfacce',
         'mercati',
         'e-commerce',
         'sistemi',
         'modelli',
         'schemi',
         'reti',
         'applicazioni',
         'metriche',
         'e-business',
         'funzionalità',
         'esperienze',
         'webservices',
         'metodologie'),
        ('implementate',
         'utilizzo',
         'integrate',
         'ottimali',
         'evolutive',
         'abilitate',
         'reinventate',
         'aggregate',
         'migliorate',
         'incentivate',
         'monetizzate',
         'sinergizzate',
         'strategiche',
         'deploy',
         'marchi',
         'accrescitive',
         'target',
         'sintetizzate',
         'spedizioni',
         'massimizzate',
         'innovazione',
         'guida',
         'estensioni',
         'generate',
         'exploit',
         'transizionali',
         'matrici',
         'ricontestualizzate'),
        ('valore aggiunto',
         'verticalizzate',
         'proattive',
         'forti',
         'rivoluzionari',
         'scalabili',
         'innovativi',
         'intuitivi',
         'strategici',
         'e-business',
         'mission-critical',
         '24/7',
         'globali',
         'B2B',
         'B2C',
         'granulari',
         'virtuali',
         'virali',
         'dinamiche',
         'magnetiche',
         'web',
         'interattive',
         'sexy',
         'back-end',
         'real-time',
         'efficienti',
         'front-end',
         'distributivi',
         'estensibili',
         'mondiali',
         'open-source',
         'cross-platform',
         'sinergiche',
         'out-of-the-box',
         'enterprise',
         'integrate',
         'di impatto',
         'wireless',
         'trasparenti',
         'next-generation',
         'cutting-edge',
         'visionari',
         'plug-and-play',
         'collaborative',
         'olistiche',
         'ricche'))

    company_suffixes = ('SPA', 'e figli', 'Group', 's.r.l.')

    def catch_phrase(self):
        """
        :example 'Robust full-range hub'
        """
        result = []
        for word_list in self.catch_phrase_words:
            result.append(self.random_element(word_list))

        return " ".join(result)

    def bs(self):
        """
        :example 'integrate extensible convergence'
        """
        result = []
        for word_list in self.bsWords:
            result.append(self.random_element(word_list))

        return " ".join(result)

    def _random_vat_office(self):
        """
        Returns a random code identifying the VAT office needed to build a valid VAT with company_vat.

        See https://it.wikipedia.org/wiki/Partita_IVA#Tabella_degli_Uffici_IVA
        """
        val = self.random_int(1, 104)

        # handle special cases
        if val == 101:
            return 120
        elif val == 102:
            return 121
        elif val == 103:
            return 888
        elif val == 104:
            return 999
        # else: between 1 and 100 are all valid
        return val

    def company_vat(self):
        """
        Returns Italian VAT identification number (Partita IVA).
        """
        code = self.bothify('#######') + str(self._random_vat_office()).zfill(3)
        luhn_checksum = str(calculate_luhn(code))
        return f'IT{code}{luhn_checksum}'



class BaseFakerException(Exception):
    """The base exception for all Faker exceptions."""


class UniquenessException(BaseFakerException):
    """To avoid infinite loops, after a certain number of attempts,
    the "unique" attribute of the Proxy will throw this exception.
    """



import re

from string import ascii_uppercase

from .. import BaseProvider

localized = True


class Provider(BaseProvider):
    """Implement default automotive provider for Faker."""

    license_formats = ()

    def license_plate(self):
        """Generate a license plate."""
        temp = re.sub(r'\?',
                      lambda x: self.random_element(ascii_uppercase),
                      self.random_element(self.license_formats))
        return self.numerify(temp)



from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``hu_HU`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Hungary
    """

    license_formats = (
        '???-###',
    )



from .. import Provider as LoremProvider


class Provider(LoremProvider):
    """Implement lorem provider for ``ru_RU`` locale."""

    word_list = (
        'войти', 'монета', 'вскинуть', 'желание', 'экзамен', 'налоговый',
        'вытаскивать', 'приятель', 'вздрагивать', 'куча', 'порт', 'точно',
        'заплакать', 'изба', 'правление', 'художественный', 'мучительно',
        'изображать', 'фонарик', 'миф', 'грустный', 'опасность', 'мера',
        'пастух', 'факультет', 'мелькнуть', 'полевой', 'другой', 'выраженный',
        'забирать', 'рот', 'народ', 'соответствие', 'тута', 'коммунизм',
        'решение', 'плод', 'собеседник', 'возмутиться', 'достоинство',
        'господь', 'болото', 'инфекция', 'голубчик', 'сынок', 'пространство',
        'прощение', 'прежде', 'хотеть', 'ленинград', 'даль', 'развитый',
        'близко', 'более', 'спорт', 'эпоха', 'ответить', 'освободить', 'совет',
        'проход', 'палец', 'вчера', 'приличный', 'ярко', 'белье', 'кузнец',
        'неожиданно', 'вперед', 'зато', 'кольцо', 'передо', 'мгновение',
        'плавно', 'табак', 'число', 'изучить', 'тяжелый', 'рассуждение',
        'салон', 'идея', 'что', 'светило', 'порода', 'сомнительный', 'бок',
        'очко', 'неудобно', 'советовать', 'отдел', 'помолчать', 'поздравлять',
        'пробовать', 'дошлый', 'смеяться', 'упорно', 'вздрогнуть', 'затянуться',
        'танцевать', 'песенка', 'выбирать', 'правильный', 'намерение', 'издали',
        'запустить', 'наслаждение', 'крыса', 'лететь', 'космос', 'радость',
        'поезд', 'находить', 'гулять', 'горький', 'бочок', 'ночь', 'счастье',
        'уничтожение', 'дьявол', 'коробка', 'спасть', 'кожа', 'провинция',
        'прелесть', 'тюрьма', 'низкий', 'сверкать', 'темнеть', 'солнце',
        'дружно', 'настать', 'блин', 'степь', 'самостоятельно', 'крутой',
        'картинка', 'зачем', 'рабочий', 'необычный', 'армейский', 'труп',
        'ягода', 'около', 'монета', 'естественный', 'юный', 'район', 'скрытый',
        'поймать', 'строительство', 'палата', 'миг', 'триста', 'штаб', 'ломать',
        'возможно', 'полюбить', 'человечек', 'легко', 'чувство', 'ручей',
        'карман', 'деньги', 'неправда', 'сравнение', 'грудь', 'отъезд',
        'возникновение', 'степь', 'возбуждение', 'деловой', 'следовательно',
        'жидкий', 'сынок', 'художественный', 'поколение', 'расстегнуть', 'пища',
        'ученый', 'секунда', 'успокоиться', 'вряд', 'аж', 'вскакивать', 'мимо',
        'падать', 'потянуться', 'угроза', 'растеряться', 'бегать', 'стакан',
        'о', 'кпсс', 'ныне', 'пол', 'реклама', 'при', 'школьный', 'премьера',
        'дальний', 'потрясти', 'освобождение', 'покидать', 'наступать', 'жить',
        'какой', 'обида', 'командование', 'девка', 'выражаться', 'головной',
        'второй', 'князь', 'социалистический', 'головка', 'привлекать', 'через',
        'господь', 'результат', 'отметить', 'ведь', 'падаль', 'покидать',
        'художественный', 'правый', 'висеть', 'лапа', 'каюта', 'слишком',
        'нервно', 'серьезный', 'зима', 'заработать', 'эффект', 'пропасть',
        'плод', 'что', 'висеть', 'холодно', 'единый', 'выкинуть', 'мрачно',
        'выгнать', 'умирать', 'иной', 'космос', 'природа', 'функция',
        'поставить', 'оборот', 'услать', 'очередной', 'медицина', 'функция',
        'зарплата', 'выдержать', 'расстройство', 'адвокат', 'задержать',
        'появление', 'инвалид', 'интеллектуальный', 'исследование', 'господь',
        'смертельный', 'спичка', 'вариант', 'рай', 'одиннадцать', 'чем',
        'манера', 'магазин', 'поговорить', 'полоска', 'помимо', 'построить',
        'домашний', 'механический', 'сохранять', 'отражение', 'научить',
        'тесно', 'аллея', 'прежний', 'посидеть', 'славный', 'очутиться',
        'лететь', 'невозможно', 'порядок', 'выразить', 'спешить', 'сынок',
        'ребятишки', 'угроза', 'оставить', 'цвет', 'налево', 'парень',
        'миллиард', 'горький', 'трубка', 'подробность', 'пасть', 'непривычный',
        'угодный', 'засунуть', 'цель', 'запретить', 'дремать', 'разуметься',
        'приходить', 'совещание', 'постоянный', 'анализ', 'терапия', 'приятель',
        'процесс', 'академик', 'металл', 'развернуться', 'жестокий', 'интернет',
        'банда', 'изменение', 'коллектив', 'похороны', 'устройство',
        'торопливый', 'разводить', 'промолчать', 'подземный', 'пламя',
        'редактор', 'теория', 'карандаш', 'упор', 'означать', 'бабочка',
        'четыре', 'столетие', 'разнообразный', 'витрина', 'нож', 'команда',
        'шлем', 'недостаток', 'протягивать', 'за', 'металл', 'добиться',
        'сутки', 'четко', 'предоставить', 'тысяча', 'запеть', 'бригада',
        'мелочь', 'выраженный', 'пересечь', 'сходить', 'вообще', 'рис', 'банк',
        'бак', 'передо', 'назначить', 'важный', 'правление', 'палка', 'трясти',
        'уронить', 'витрина', 'основание', 'да', 'перебивать', 'дыхание',
        'применяться', 'июнь', 'бетонный', 'избегать', 'умолять', 'мягкий',
        'заявление', 'конференция', 'встать', 'свежий', 'сопровождаться',
        'цепочка', 'выражение', 'угол', 'ботинок', 'ложиться', 'инструкция',
        'присесть', 'решетка', 'еврейский', 'порог', 'зеленый', 'граница',
        'ставить', 'смелый', 'сустав', 'роса', 'демократия', 'вывести',
        'конструкция', 'задрать', 'багровый', 'военный', 'направо', 'житель',
        'товар', 'неправда', 'материя', 'командующий', 'кидать', 'заложить',
        'лиловый', 'слать', 'горький', 'пространство', 'провал', 'мусор',
        'наткнуться', 'торговля', 'монета', 'место', 'спалить', 'бровь',
        'левый', 'хлеб', 'коричневый', 'потом', 'страсть', 'виднеться',
        'роскошный', 'способ', 'костер', 'заведение', 'пропадать', 'слишком',
        'пятеро', 'мальчишка', 'тусклый', 'неожиданный', 'плясать', 'дурацкий',
        'дрогнуть', 'сбросить', 'прошептать', 'беспомощный', 'рота', 'песня',
        'тревога', 'некоторый', 'термин', 'нажать', 'видимо', 'валюта', 'набор',
        'боец', 'райком', 'новый', 'скользить', 'руководитель', 'волк',
        'изредка', 'понятный', 'пропаганда', 'остановить', 'исполнять', 'ход',
        'госпожа', 'печатать', 'командир', 'снимать', 'казнь', 'невыносимый',
        'спорт', 'тревога', 'уточнить', 'актриса', 'полностью', 'покинуть',
        'сверкающий', 'мотоцикл', 'дорогой', 'указанный', 'ремень', 'посвятить',
        'один', 'а', 'доставать', 'хозяйка', 'носок', 'написать', 'еврейский',
        'призыв', 'увеличиваться', 'равнодушный',
    )



from math import ceil

from .. import Provider as BaseProvider


class Provider(BaseProvider):
    """
    A Faker provider for the Slovakian VAT IDs
    """

    vat_id_formats = (
        'SK##########',
    )

    national_id_months = ['%.2d' % i for i in range(1, 13)] + ['%.2d' % i for i in range(51, 63)]

    def vat_id(self):
        """
        http://ec.europa.eu/taxation_customs/vies/faq.html#item_11
        :return: a random Slovakian VAT ID
        """

        return self.bothify(self.random_element(self.vat_id_formats))

    def birth_number(self):
        """
        Birth Number (Czech/Slovak: rodné číslo (RČ))
        https://en.wikipedia.org/wiki/National_identification_number#Czech_Republic_and_Slovakia
        """
        birthdate = self.generator.date_of_birth()
        year = f'{birthdate:%y}'
        month = self.random_element(self.national_id_months)
        day = f'{birthdate:%d}'
        if birthdate.year > 1953:
            sn = self.random_number(4, True)
        else:
            sn = self.random_number(3, True)
        number = int(f'{year}{month}{day}{sn}')
        birth_number = str(ceil(number / 11) * 11)
        if year == '00':
            birth_number = '00' + birth_number
        elif year[0] == '0':
            birth_number = '0' + birth_number
        return f'{birth_number[:6]}/{birth_number[6::]}'



from .. import Provider as InternetProvider


class Provider(InternetProvider):
    safe_email_tlds = ('org', 'com', 'net', 'ch')
    free_email_domains = (
        'gmail.com',
        'hotmail.fr',
        'yahoo.fr',
        'bluewin.ch',
        'romandie.com',
        'hispeed.ch',
        'sunrise.ch',
        'vtxnet.ch')
    tlds = ('com', 'com', 'com', 'net', 'org', 'ch', 'ch', 'ch')

    replacements = (
        ('ä', 'ae'), ('à', 'a'), ('â', 'a'),
        ('ç', 'c'),
        ('é', 'e'), ('è', 'e'), ('ê', 'e'), ('ë', 'e'),
        ('ï', 'i'), ('î', 'i'),
        ('ö', 'oe'), ('ô', 'o'),
        ('ü', 'ue'), ('ù', 'u'), ('ü', 'u'),
        ('ß', 'ss'),
    )



from .. import Provider as AddressProvider


class Provider(AddressProvider):
    # City and States names taken from wikipedia
    # Street format taken from some common famous places in India
    # Link for cities: https://en.wikipedia.org/wiki/List_of_cities_in_India_by_population
    # Link for States: https://en.wikipedia.org/wiki/States_and_union_territories_of_India
    # Links for street name formats: https://www.mumbai77.com/city/3313/travel/old-new-street-names/

    city_formats = ('{{city_name}}', )

    street_name_formats = (
        '{{last_name}} Nagar',
        '{{last_name}} Zila',
        '{{last_name}} Street',
        '{{last_name}} Ganj',
        '{{last_name}} Road',
        '{{last_name}} Path',
        '{{last_name}} Marg',
        '{{last_name}} Chowk',
        '{{last_name}} Circle',
        '{{last_name}}',
    )

    street_address_formats = (
        '{{building_number}}, {{street_name}}',
        '{{building_number}}\n{{street_name}}',
    )

    address_formats = (
        '{{street_address}}\n{{city}} {{postcode}}',
        '{{street_address}}\n{{city}}-{{postcode}}',
        '{{street_address}}, {{city}} {{postcode}}',
        '{{street_address}}, {{city}}-{{postcode}}',
    )

    building_number_formats = (
        'H.No. ###', '###', 'H.No. ##', '##', '##/##', "##/###")

    postcode_formats = ('######', )

    cities = (
        'Mumbai',
        'Delhi',
        'Kolkata',
        'Chennai',
        'Bangalore',
        'Hyderabad',
        'Ahmedabad',
        'Kanpur',
        'Pune',
        'Surat',
        'Jaipur',
        'Lucknow',
        'Nagpur',
        'Indore',
        'Bhopal',
        'Ludhiana',
        'Patna',
        'Visakhapatnam',
        'Vadodara',
        'Agra',
        'Thane',
        'Kalyan-Dombivli',
        'Varanasi',
        'Ranchi',
        'Nashik',
        'Dhanbad',
        'Faridabad',
        'Meerut',
        'Pimpri-Chinchwad',
        'Howrah',
        'Allahabad',
        'Ghaziabad',
        'Rajkot',
        'Amritsar',
        'Jabalpur',
        'Coimbatore',
        'Madurai',
        'Srinagar',
        'Aurangabad',
        'Solapur',
        'Vijayawada',
        'Jodhpur',
        'Gwalior',
        'Guwahati',
        'Chandigarh',
        'Hubli–Dharwad',
        'Mysore',
        'Tiruchirappalli',
        'Bareilly',
        'Jalandhar',
        'Navi Mumbai',
        'Salem',
        'Kota',
        'Vasai-Virar',
        'Aligarh',
        'Moradabad',
        'Bhubaneswar',
        'Gorakhpur',
        'Raipur',
        'Bhiwandi',
        'Kochi',
        'Jamshedpur',
        'Bhilai',
        'Amravati',
        'Cuttack',
        'Warangal',
        'Bikaner',
        'Mira-Bhayandar',
        'Guntur',
        'Bhavnagar',
        'Durgapur',
        'Kolhapur',
        'Ajmer',
        'Asansol',
        'Ulhasnagar',
        'Siliguri',
        'Jalgaon',
        'Saharanpur',
        'Jamnagar',
        'Bhatpara',
        'Sangli-Miraj & Kupwad',
        'Kozhikode',
        'Nanded',
        'Ujjain',
        'Dehradun',
        'Rourkela',
        'Gulbarga',
        'Tirunelveli',
        'Malegaon',
        'Akola',
        'Belgaum',
        'Mangalore',
        'Bokaro',
        'South Dumdum',
        'Udaipur',
        'Gaya',
        'Maheshtala',
        'Jhansi',
        'Nellore',
        'Jammu',
        'Thiruvananthapuram',
        'Davanagere',
        'Kollam',
        'Panihati',
        'Kurnool',
        'Tiruppur',
        'Dhule',
        'Bhagalpur',
        'Rajpur Sonarpur',
        'Kakinada',
        'Thrissur',
        'Bellary',
        'Muzaffarnagar',
        'Korba',
        'Rajahmundry',
        'Kamarhati',
        'Ambattur',
        'Berhampur',
        'Ahmednagar',
        'Muzaffarpur',
        'Noida',
        'Patiala',
        'Mathura',
        'New Delhi',
        'Latur',
        'Sambalpur',
        'Shahjahanpur',
        'Kulti',
        'Chandrapur',
        'Nizamabad',
        'Rohtak',
        'Bardhaman',
        'Rampur',
        'Bhilwara',
        'Firozabad',
        'Bilaspur',
        'Shimoga',
        'Agartala',
        'Gopalpur',
        'Darbhanga',
        'Panipat',
        'Bally',
        'Alwar',
        'Parbhani',
        'Ichalkaranji',
        'Anantapuram',
        'Baranagar',
        'Tumkur',
        'Ramagundam',
        'Jalna',
        'Durg',
        'Sagar',
        'Bihar Sharif',
        'Dewas',
        'Barasat',
        'Avadi',
        'Farrukhabad',
        'Aizawl',
        'Tirupati',
        'Bijapur',
        'Satara',
        'Satna',
        'Ratlam',
        'Imphal',
        'Pondicherry',
        'North Dumdum',
        'Anantapur',
        'Khammam',
        'Ozhukarai',
        'Bathinda',
        'Thoothukudi',
        'Thanjavur',
        'Naihati',
        'Sonipat',
        'Mau',
        'Tiruvottiyur',
        'Hapur',
        'Sri Ganganagar',
        'Karnal',
        'Etawah',
        'Nagercoil',
        'Raichur',
        'Raurkela Industrial Township',
        'Secunderabad',
        'Karimnagar',
        'Mirzapur',
        'Bharatpur',
        'Ambarnath',
        'Arrah',
        'Uluberia',
        'Serampore',
        'Dindigul',
        'Gandhinagar',
        'Burhanpur',
        'Nadiad',
        'Eluru',
        'Yamunanagar',
        'Kharagpur',
        'Munger',
        'Pali',
        'Katni',
        'Singrauli',
        'Tenali',
        'Sikar',
        'Silchar',
        'Rewa',
        'Sambhal',
        'Machilipatnam',
        'Vellore',
        'Alappuzha',
        'Bulandshahr',
        'Haridwar',
        'Vijayanagaram',
        'Erode',
        'Gurgaon',
        'Bidar',
        'Bhusawal',
        'Khandwa',
        'Purnia',
        'Haldia',
        'Chinsurah',
        'Bhiwani',
        'Raebareli',
        'Junagadh',
        'Bahraich',
        'Gandhidham',
        'Mango',
        'Raiganj',
        'Amroha',
        'Sultan Pur Majra',
        'Hospet',
        'Bidhannagar',
        'Malda',
        'Sirsa',
        'Berhampore',
        'Jaunpur',
        'Surendranagar Dudhrej',
        'Madhyamgram',
        'Kirari Suleman Nagar',
        'Bhind',
        'Nandyal',
        'Chittoor',
        'Bhalswa Jahangir Pur',
        'Fatehpur',
        'Morena',
        'Nangloi Jat',
        'Ongole',
        'Karawal Nagar',
        'Shivpuri',
        'Morbi',
        'Unnao',
        'Pallavaram',
        'Kumbakonam',
        'Shimla',
        'Mehsana',
        'Panchkula',
        'Orai',
        'Ambala',
        'Dibrugarh',
        'Guna',
        'Danapur',
        'Sasaram',
        'Anand',
        'Kottayam',
        'Hazaribagh',
        'Kadapa',
        'Saharsa',
        'Nagaon',
        'Loni',
        'Hajipur',
        'Dehri',
        'Bettiah',
        'Katihar',
        'Deoghar',
        'Jorhat',
        'Siwan',
        'Panvel',
        'Hosur',
        'Tinsukia',
        'Bongaigaon',
        'Motihari',
        'Jamalpur',
        'Suryapet',
        'Begusarai',
        'Miryalaguda',
        'Proddatur',
        'Karaikudi',
        'Kishanganj',
        'Phusro',
        'Buxar',
        'Tezpur',
        'Jehanabad',
        'Aurangabad',
        'Chapra',
        'Ramgarh',
        'Gangtok',
        'Adoni',
        'Amaravati',
        'Ballia',
        'Bhimavaram',
        'Dharmavaram',
        'Giridih',
        'Gudivada',
        'Guntakal',
        'Hindupur',
        'Kavali',
        'Khora ',
        'Ghaziabad',
        'Madanapalle',
        'Mahbubnagar',
        'Medininagar',
        'Narasaraopet',
        'Phagwara',
        'Pudukkottai',
        'Srikakulam',
        'Tadepalligudem',
        'Tadipatri',
        'Udupi',
    )

    states = (
        'Andhra Pradesh',
        'Arunachal Pradesh',
        'Assam',
        'Bihar',
        'Chhattisgarh',
        'Goa',
        'Gujarat',
        'Haryana',
        'Himachal Pradesh',
        'Jharkhand',
        'Karnataka',
        'Kerala',
        'Madhya Pradesh',
        'Maharashtra',
        'Manipur',
        'Meghalaya',
        'Mizoram',
        'Nagaland',
        'Odisha',
        'Punjab',
        'Rajasthan',
        'Sikkim',
        'Tamil Nadu',
        'Telangana',
        'Tripura',
        'Uttar Pradesh',
        'Uttarakhand',
        'West Bengal',
    )

    def city_name(self):
        return self.random_element(self.cities)

    def state(self):
        return self.random_element(self.states)



from .. import Provider as InternetProvider


class Provider(InternetProvider):
    safe_email_tlds = ('com', 'net', 'pt', 'pt')
    free_email_domains = ('gmail.com', 'hotmail.com', 'clix.pt', 'sapo.pt')
    tlds = ('com', 'com', 'com', 'net', 'org', 'pt', 'pt', 'pt')



import warnings

from ..fr_CA import Provider as FRCAProvider


class Provider(FRCAProvider):
    def __init__(self, *args, **kwargs):
        warnings.warn("fr_QC locale is deprecated. Please use fr_CA.")
        super().__init__(*args, **kwargs)



from .. import BaseProvider

localized = True


class Provider(BaseProvider):
    """Implement default barcode provider for Faker.

    Sources:

    - https://gs1.org/standards/id-keys/company-prefix
    """

    local_prefixes = ()

    def _ean(self, length=13, prefixes=()):
        if length not in (8, 13):
            raise AssertionError("length can only be 8 or 13")

        code = [self.random_digit() for _ in range(length - 1)]

        if prefixes:
            prefix = self.random_element(prefixes)
            code[:len(prefix)] = map(int, prefix)

        if length == 8:
            weights = [3, 1, 3, 1, 3, 1, 3]
        elif length == 13:
            weights = [1, 3, 1, 3, 1, 3, 1, 3, 1, 3, 1, 3]

        weighted_sum = sum(x * y for x, y in zip(code, weights))
        check_digit = (10 - weighted_sum % 10) % 10
        code.append(check_digit)

        return ''.join(str(x) for x in code)

    def ean(self, length=13, prefixes=()):
        """Generate an EAN barcode of the specified ``length``.

        The value of ``length`` can only be ``8`` or ``13`` (default) which will
        create an EAN-8 or an EAN-13 barcode respectively.

        If a value for ``prefixes`` is specified, the result will begin with one
        of the sequences in ``prefixes``.

        :sample: length=13
        :sample: length=8
        :sample: prefixes=('00',)
        :sample: prefixes=('45', '49')
        """
        return self._ean(length, prefixes=prefixes)

    def ean8(self, prefixes=()):
        """Generate an EAN-8 barcode.

        This method uses |ean| under the hood with the ``length`` argument
        explicitly set to ``8``.

        If a value for ``prefixes`` is specified, the result will begin with one
        of the sequences in ``prefixes``.

        :sample:
        :sample: prefixes=('00',)
        :sample: prefixes=('45', '49')
        """
        return self._ean(8, prefixes=prefixes)

    def ean13(self, prefixes=()):
        """Generate an EAN-13 barcode.

        This method uses |ean| under the hood with the ``length`` argument
        explicitly set to ``13``.

        If a value for ``prefixes`` is specified, the result will begin with one
        of the sequences in ``prefixes``.

        .. note::
           Codes starting with a leading zero are treated specially in some
           barcode readers. For more information on compatibility with UPC-A
           codes, see |EnUsBarcodeProvider.ean13|.

        :sample:
        :sample: prefixes=('00',)
        :sample: prefixes=('45', '49')
        """
        return self._ean(13, prefixes=prefixes)

    def localized_ean(self, length=13):
        """Generate a localized EAN barcode of the specified ``length``.

        The value of ``length`` can only be ``8`` or ``13`` (default) which will
        create an EAN-8 or an EAN-13 barcode respectively.

        This method uses the standard barcode provider's |ean| under the hood
        with the ``prefixes`` argument explicitly set to ``local_prefixes`` of
        a localized barcode provider implementation.

        :sample:
        :sample: length=13
        :sample: length=8
        """
        return self._ean(length, prefixes=self.local_prefixes)

    def localized_ean8(self):
        """Generate a localized EAN-8 barcode.

        This method uses |localized_ean| under the hood with the ``length``
        argument explicitly set to ``8``.
        """
        return self.localized_ean(8)

    def localized_ean13(self):
        """Generate a localized EAN-13 barcode.

        This method uses |localized_ean| under the hood with the ``length``
        argument explicitly set to ``13``.
        """
        return self.localized_ean(13)



from .. import Provider as CompanyProvider


def company_id_checksum(digits):
    digits = list(digits)
    weights = 6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2

    dv = sum(w * d for w, d in zip(weights[1:], digits))
    dv = (11 - dv) % 11
    dv = 0 if dv >= 10 else dv
    digits.append(dv)

    dv2 = sum(w * d for w, d in zip(weights, digits))
    dv2 = (11 - dv2) % 11
    dv2 = 0 if dv2 >= 10 else dv2
    digits.append(dv2)

    return digits[-2:]


class Provider(CompanyProvider):
    formats = (
        '{{last_name}} {{company_suffix}}',
        '{{last_name}} {{last_name}} {{company_suffix}}',
        '{{last_name}}',
        '{{last_name}}',
    )

    catch_phrase_formats = (
        '{{catch_phrase_noun}} {{catch_phrase_verb}} {{catch_phrase_attribute}}', )

    nouns = (
        'a segurança',
        'o prazer',
        'o conforto',
        'a simplicidade',
        'a certeza',
        'a arte',
        'o poder',
        'o direito',
        'a possibilidade',
        'a vantagem',
        'a liberdade')

    verbs = (
        'de conseguir',
        'de avançar',
        'de evoluir',
        'de mudar',
        'de inovar',
        'de ganhar',
        'de atingir seus objetivos',
        'de concretizar seus projetos',
        'de realizar seus sonhos')

    attributes = (
        'de maneira eficaz',
        'mais rapidamente',
        'mais facilmente',
        'simplesmente',
        'com toda a tranquilidade',
        'antes de tudo',
        'naturalmente',
        'sem preocupação',
        'em estado puro',
        'com força total',
        'direto da fonte',
        'com confiança')

    company_suffixes = ('S/A', 'S.A.', 'Ltda.', '- ME', '- EI', 'e Filhos')

    def catch_phrase_noun(self):
        """
        Returns a random catch phrase noun.
        """
        return self.random_element(self.nouns)

    def catch_phrase_attribute(self):
        """
        Returns a random catch phrase attribute.
        """
        return self.random_element(self.attributes)

    def catch_phrase_verb(self):
        """
        Returns a random catch phrase verb.
        """
        return self.random_element(self.verbs)

    def catch_phrase(self):
        """
        :example 'a segurança de evoluir sem preocupação'
        """
        pattern = self.random_element(self.catch_phrase_formats)
        catch_phrase = self.generator.parse(pattern)
        catch_phrase = catch_phrase[0].upper() + catch_phrase[1:]
        return catch_phrase

    def company_id(self):
        digits = self.random_sample(range(10), 8) + [0, 0, 0, 1]
        digits += company_id_checksum(digits)
        return ''.join(str(d) for d in digits)

    def cnpj(self):
        digits = self.company_id()
        return f'{digits[:2]}.{digits[2:5]}.{digits[5:8]}/{digits[8:12]}-{digits[12:]}'



from .. import Provider as InternetProvider


class Provider(InternetProvider):
    tlds = (
        # From https://en.wikipedia.org/wiki/List_of_Internet_top-level_domains
        'com', 'org', 'net', 'int', 'edu', 'gov', 'mil',

        # From https://id.wikipedia.org/wiki/.id
        'id', 'ac.id', 'biz.id', 'co.id', 'desa.id', 'go.id', 'mil.id',
        'my.id', 'net.id', 'or.id', 'ponpes.id', 'sch.id', 'web.id',
    )



from .. import Provider as CurrencyProvider


class Provider(CurrencyProvider):

    price_formats = ["#.##", "%#.##", "%##.##", "%,###.##", "%#,###.##"]

    def pricetag(self):
        return (
            "$\N{no-break space}"
            + self.numerify(self.random_element(self.price_formats))
        )



import string

from datetime import datetime

from .. import BaseProvider


class Provider(BaseProvider):
    """Implement default user agent provider for Faker."""

    user_agents = (
        'chrome', 'firefox', 'internet_explorer', 'opera', 'safari',
    )

    windows_platform_tokens = (
        'Windows 95', 'Windows 98', 'Windows 98; Win 9x 4.90', 'Windows CE',
        'Windows NT 4.0', 'Windows NT 5.0', 'Windows NT 5.01',
        'Windows NT 5.1', 'Windows NT 5.2', 'Windows NT 6.0', 'Windows NT 6.1',
        'Windows NT 6.2',
    )

    linux_processors = ('i686', 'x86_64')

    mac_processors = ('Intel', 'PPC', 'U; Intel', 'U; PPC')

    android_versions = (
        '1.0', '1.1', '1.5', '1.6', '2.0', '2.0.1', '2.1', '2.2', '2.2.1', '2.2.2', '2.2.3', '2.3', '2.3.1', '2.3.2',
        '2.3.3', '2.3.4', '2.3.5', '2.3.6', '2.3.7', '3.0', '3.1', '3.2', '3.2.1', '3.2.2', '3.2.3', '3.2.4', '3.2.5',
        '3.2.6', '4.0', '4.0.1', '4.0.2', '4.0.3', '4.0.4', '4.1', '4.1.1', '4.1.2', '4.2', '4.2.1', '4.2.2', '4.3',
        '4.3.1', '4.4', '4.4.1', '4.4.2', '4.4.3', '4.4.4', '5.0', '5.0.1', '5.0.2', '5.1', '5.1.1', '6.0', '6.0.1',
        '7.0', '7.1', '7.1.1', '7.1.2', '8.0.0', '8.1.0', '9',
    )

    apple_devices = ('iPhone', 'iPad')

    ios_versions = (
        '3.1.3', '4.2.1', '5.1.1', '6.1.6', '7.1.2', '9.3.5', '9.3.6', '10.3.3', '10.3.4', '12.4',
    )

    def mac_processor(self):
        """Generate a MacOS processor token used in user agent strings."""
        return self.random_element(self.mac_processors)

    def linux_processor(self):
        """Generate a Linux processor token used in user agent strings."""
        return self.random_element(self.linux_processors)

    def user_agent(self):
        """Generate a random web browser user agent string."""
        name = self.random_element(self.user_agents)
        return getattr(self, name)()

    def chrome(self, version_from=13, version_to=63,
               build_from=800, build_to=899):
        """Generate a Chrome web browser user agent string."""
        saf = f'{self.generator.random.randint(531, 536)}.{self.generator.random.randint(0, 2)}'
        bld = self.lexify(self.numerify('##?###'), string.ascii_uppercase)
        tmplt = '({0}) AppleWebKit/{1} (KHTML, like Gecko)' \
                ' Chrome/{2}.0.{3}.0 Safari/{4}'
        tmplt_ios = '({0}) AppleWebKit/{1} (KHTML, like Gecko)' \
                    ' CriOS/{2}.0.{3}.0 Mobile/{4} Safari/{1}'
        platforms = (
            tmplt.format(self.linux_platform_token(),
                         saf,
                         self.generator.random.randint(version_from, version_to),
                         self.generator.random.randint(build_from, build_to),
                         saf),
            tmplt.format(self.windows_platform_token(),
                         saf,
                         self.generator.random.randint(version_from, version_to),
                         self.generator.random.randint(build_from, build_to),
                         saf),
            tmplt.format(self.mac_platform_token(),
                         saf,
                         self.generator.random.randint(version_from, version_to),
                         self.generator.random.randint(build_from, build_to),
                         saf),
            tmplt.format('Linux; {}'.format(self.android_platform_token()),
                         saf,
                         self.generator.random.randint(version_from, version_to),
                         self.generator.random.randint(build_from, build_to),
                         saf),
            tmplt_ios.format(self.ios_platform_token(),
                             saf,
                             self.generator.random.randint(version_from, version_to),
                             self.generator.random.randint(build_from, build_to),
                             bld),
        )

        return 'Mozilla/5.0 ' + self.random_element(platforms)

    def firefox(self):
        """Generate a Mozilla Firefox web browser user agent string."""
        ver = (
            (f'Gecko/{self.generator.date_time_between(datetime(2011, 1, 1))} '
             f'Firefox/{self.generator.random.randint(4, 15)}.0'),
            (f'Gecko/{self.generator.date_time_between(datetime(2010, 1, 1))} '
             f'Firefox/3.6.{self.generator.random.randint(1, 20)}'),
            f'Gecko/{self.generator.date_time_between(datetime(2010, 1, 1))} Firefox/3.8',
        )
        tmplt_win = '({0}; {1}; rv:1.9.{2}.20) {3}'
        tmplt_lin = '({0}; rv:1.9.{1}.20) {2}'
        tmplt_mac = '({0}; rv:1.9.{1}.20) {2}'
        tmplt_and = '({0}; Mobile; rv:{1}.0) Gecko/{1}.0 Firefox/{1}.0'
        tmplt_ios = '({0}) AppleWebKit/{1} (KHTML, like Gecko) FxiOS/{2}.{3}.0 Mobile/{4} Safari/{1}'
        saf = '{}.{}'.format(self.generator.random.randint(531, 536),
                             self.generator.random.randint(0, 2))
        bld = self.lexify(self.numerify('##?###'), string.ascii_uppercase)
        bld2 = self.lexify(self.numerify('#?####'), string.ascii_lowercase)
        platforms = (
            tmplt_win.format(self.windows_platform_token(),
                             self.generator.locale().replace('_', '-'),
                             self.generator.random.randint(0, 2),
                             self.generator.random.choice(ver)),
            tmplt_lin.format(self.linux_platform_token(),
                             self.generator.random.randint(5, 7),
                             self.generator.random.choice(ver)),
            tmplt_mac.format(self.mac_platform_token(),
                             self.generator.random.randint(2, 6),
                             self.generator.random.choice(ver)),
            tmplt_and.format(self.android_platform_token(),
                             self.generator.random.randint(5, 68)),
            tmplt_ios.format(self.ios_platform_token(),
                             saf,
                             self.generator.random.randint(9, 18),
                             bld2,
                             bld),
        )

        return 'Mozilla/5.0 ' + self.random_element(platforms)

    def safari(self):
        """Generate a Safari web browser user agent string."""
        saf = (f'{self.generator.random.randint(531, 535)}.'
               f'{self.generator.random.randint(1, 50)}.'
               f'{self.generator.random.randint(1, 7)}')
        if not self.generator.random.getrandbits(1):
            ver = f'{self.generator.random.randint(4, 5)}.{self.generator.random.randint(0, 1)}'
        else:
            ver = f'{self.generator.random.randint(4, 5)}.0.{self.generator.random.randint(1, 5)}'
        tmplt_win = '(Windows; U; {0}) AppleWebKit/{1} (KHTML, like Gecko)' \
                    ' Version/{2} Safari/{3}'
        tmplt_mac = '({0} rv:{1}.0; {2}) AppleWebKit/{3} (KHTML, like Gecko)' \
                    ' Version/{4} Safari/{5}'
        tmplt_ipod = '(iPod; U; CPU iPhone OS {0}_{1} like Mac OS X; {2})' \
                     ' AppleWebKit/{3} (KHTML, like Gecko) Version/{4}.0.5' \
                     ' Mobile/8B{5} Safari/6{6}'
        locale = self.generator.locale().replace('_', '-')
        platforms = (
            tmplt_win.format(self.windows_platform_token(),
                             saf,
                             ver,
                             saf),
            tmplt_mac.format(self.mac_platform_token(),
                             self.generator.random.randint(2, 6),
                             locale,
                             saf,
                             ver,
                             saf),
            tmplt_ipod.format(self.generator.random.randint(3, 4),
                              self.generator.random.randint(0, 3),
                              locale,
                              saf,
                              self.generator.random.randint(3, 4),
                              self.generator.random.randint(111, 119),
                              saf),
        )

        return 'Mozilla/5.0 ' + self.random_element(platforms)

    def opera(self):
        """Generate an Opera web browser user agent string."""
        token = self.linux_platform_token() if self.generator.random.getrandbits(1) else self.windows_platform_token()
        locale = self.generator.locale().replace('_', '-')
        platform = (f'({token}; {locale}) Presto/2.9.{self.generator.random.randint(160, 190)} '
                    f'Version/{self.generator.random.randint(10, 12)}.00')
        return f'Opera/{self.generator.random.randint(8, 9)}.{self.generator.random.randint(10, 99)}.{platform}'

    def internet_explorer(self):
        """Generate an IE web browser user agent string."""
        return (f'Mozilla/5.0 (compatible; MSIE {self.generator.random.randint(5, 9)}.0; '
                f'{self.windows_platform_token()}; '
                f'Trident/{self.generator.random.randint(3, 5)}.{self.generator.random.randint(0, 1)})')

    def windows_platform_token(self):
        """Generate a Windows platform token used in user agent strings."""
        return self.random_element(self.windows_platform_tokens)

    def linux_platform_token(self):
        """Generate a Linux platform token used in user agent strings."""
        return f'X11; Linux {self.random_element(self.linux_processors)}'

    def mac_platform_token(self):
        """Generate a MacOS platform token used in user agent strings."""
        return (f'Macintosh; {self.random_element(self.mac_processors)} Mac OS X 10 '
                f'{self.generator.random.randint(5, 12)}_{self.generator.random.randint(0, 9)}')

    def android_platform_token(self):
        """Generate an Android platform token used in user agent strings."""
        return f'Android {self.random_element(self.android_versions)}'

    def ios_platform_token(self):
        """Generate an iOS platform token used in user agent strings."""
        apple_device = self.random_element(self.apple_devices)
        return (f'{apple_device}; CPU {apple_device} '
                f'OS {self.random_element(self.ios_versions).replace(".", "_")} like Mac OS X')



from .. import Provider as BaseProvider


class Provider(BaseProvider):
    # Source:
    # Turkey Republic National Number is identity number.
    # Identity number contains 11 numbers,
    # First number can't be zero
    # Eleventh number is result of division after sum first number

    def ssn(self):
        """
        :example '89340691651'
        """
        first_part = self.random_element((1, 2, 3, 4, 5, 6, 7, 8, 9))
        middle_part = self.bothify('#########')
        last_part = sum(int(x) for x in f'{first_part}{middle_part}') % 10
        return f'{first_part}{middle_part}{last_part}'



import re

from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``ar_SA`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Saudi_Arabia

    .. |license_plate_en| replace::
        :meth:`license_plate_en()`
    """

    LICENSE_FORMAT_EN = '#### ???'
    LICENSE_FORMAT_AR = '? ? ? ####'

    PLATE_CHARS_EN = 'ABDEGHJKLNRSTUVXZ'
    PLATE_CHARS_AR = 'أبدعقهحكلنرسطوىصم'

    PLATE_MAP = {
        'A': 'ا',
        'B': 'ب',
        'D': 'د',
        'E': 'ع',
        'G': 'ق',
        'H': 'ه',
        'J': 'ح',
        'K': 'ك',
        'L': 'ل',
        'N': 'ن',
        'R': 'ر',
        'S': 'س',
        'T': 'ط',
        'U': 'و',
        'V': 'ى',
        'X': 'ص',
        'Z': 'م',

        '0': '٠',
        '1': '١',
        '2': '٢',
        '3': '٣',
        '4': '٤',
        '5': '٥',
        '6': '٦',
        '7': '٧',
        '8': '٨',
        '9': '٩',
    }

    def license_plate_en(self):
        """Generate a license plate in Latin/Western characters."""
        return self.bothify(
            self.LICENSE_FORMAT_EN, letters=self.PLATE_CHARS_EN,
        )

    def license_plate_ar(self):
        """Generate a license plate in Arabic characters.

        This method first generates a license plate in Latin/Western characters
        using |license_plate_en|, and the result is translated internally to
        generate the Arabic counterpart which serves as this method's return
        value.
        """
        english_plate = self.license_plate_en()
        return self._translate_license_plate(english_plate)

    def _translate_license_plate(self, license_plate):
        nums = list(reversed(license_plate[0:4]))
        chars = list(license_plate[5:8])

        numerated = re.sub(
            r'\#',
            lambda x: self.PLATE_MAP[nums.pop()],
            self.LICENSE_FORMAT_AR,
        )
        ar_plate = re.sub(
            r'\?',
            lambda x: self.PLATE_MAP[chars.pop()],
            numerated,
        )

        return ar_plate

    def license_plate(self):
        """Generate a license plate.

        This method first generates a license plate in Latin/Western characters
        using |license_plate_en|, and the result is translated internally to
        generate the Arabic counterpart. A 2-tuple containing those results
        will serve as the return value.
        """
        en_palate = self.license_plate_en()
        ar_palate = self._translate_license_plate(en_palate)

        return en_palate, ar_palate



from .. import BaseProvider

localized = True


class Provider(BaseProvider):
    # Format: (code, name)
    currencies = (
        ("AED", "United Arab Emirates dirham"),
        ("AFN", "Afghan afghani"),
        ("ALL", "Albanian lek"),
        ("AMD", "Armenian dram"),
        ("ANG", "Netherlands Antillean guilder"),
        ("AOA", "Angolan kwanza"),
        ("ARS", "Argentine peso"),
        ("AUD", "Australian dollar"),
        ("AWG", "Aruban florin"),
        ("AZN", "Azerbaijani manat"),
        ("BAM", "Bosnia and Herzegovina convertible mark"),
        ("BBD", "Barbadian dollar"),
        ("BDT", "Bangladeshi taka"),
        ("BGN", "Bulgarian lev"),
        ("BHD", "Bahraini dinar"),
        ("BIF", "Burundian franc"),
        ("BMD", "Bermudian dollar"),
        ("BND", "Brunei dollar"),
        ("BOB", "Bolivian boliviano"),
        ("BRL", "Brazilian real"),
        ("BSD", "Bahamian dollar"),
        ("BTN", "Bhutanese ngultrum"),
        ("BWP", "Botswana pula"),
        ("BYR", "Belarusian ruble"),
        ("BZD", "Belize dollar"),
        ("CAD", "Canadian dollar"),
        ("CDF", "Congolese franc"),
        ("CHF", "Swiss franc"),
        ("CLP", "Chilean peso"),
        ("CNY", "Renminbi"),
        ("COP", "Colombian peso"),
        ("CRC", "Costa Rican colón"),
        ("CUC", "Cuban convertible peso"),
        ("CUP", "Cuban peso"),
        ("CVE", "Cape Verdean escudo"),
        ("CZK", "Czech koruna"),
        ("DJF", "Djiboutian franc"),
        ("DKK", "Danish krone"),
        ("DOP", "Dominican peso"),
        ("DZD", "Algerian dinar"),
        ("EGP", "Egyptian pound"),
        ("ERN", "Eritrean nakfa"),
        ("ETB", "Ethiopian birr"),
        ("EUR", "Euro"),
        ("FJD", "Fijian dollar"),
        ("FKP", "Falkland Islands pound"),
        ("GBP", "Pound sterling"),
        ("GEL", "Georgian lari"),
        ("GGP", "Guernsey pound"),
        ("GHS", "Ghanaian cedi"),
        ("GIP", "Gibraltar pound"),
        ("GMD", "Gambian dalasi"),
        ("GNF", "Guinean franc"),
        ("GTQ", "Guatemalan quetzal"),
        ("GYD", "Guyanese dollar"),
        ("HKD", "Hong Kong dollar"),
        ("HNL", "Honduran lempira"),
        ("HRK", "Croatian kuna"),
        ("HTG", "Haitian gourde"),
        ("HUF", "Hungarian forint"),
        ("IDR", "Indonesian rupiah"),
        ("ILS", "Israeli new shekel"),
        ("NIS", "Israeli new shekel"),
        ("IMP", "Manx pound"),
        ("INR", "Indian rupee"),
        ("IQD", "Iraqi dinar"),
        ("IRR", "Iranian rial"),
        ("ISK", "Icelandic króna"),
        ("JEP", "Jersey pound"),
        ("JMD", "Jamaican dollar"),
        ("JOD", "Jordanian dinar"),
        ("JPY", "Japanese yen"),
        ("KES", "Kenyan shilling"),
        ("KGS", "Kyrgyzstani som"),
        ("KHR", "Cambodian riel"),
        ("KMF", "Comorian franc"),
        ("KPW", "North Korean won"),
        ("KRW", "Western Krahn language"),
        ("KWD", "Kuwaiti dinar"),
        ("KYD", "Cayman Islands dollar"),
        ("KZT", "Kazakhstani tenge"),
        ("LAK", "Lao kip"),
        ("LBP", "Lebanese pound"),
        ("LKR", "Sri Lankan rupee"),
        ("LRD", "Liberian dollar"),
        ("LSL", "Lesotho loti"),
        ("LTL", "Lithuanian litas"),
        ("LYD", "Libyan dinar"),
        ("MAD", "Moroccan dirham"),
        ("MDL", "Moldovan leu"),
        ("MGA", "Malagasy ariar"),
        ("MKD", "Macedonian denar"),
        ("MMK", "Burmese kyat"),
        ("MNT", "Mongolian tugrik"),
        ("MOP", "Macanese pataca"),
        ("MRO", "Mauritanian ouguiya"),
        ("MUR", "Mauritian rupee"),
        ("MVR", "Maldivian rufiyaa"),
        ("MWK", "Malawian kwacha"),
        ("MXN", "Mexican peso"),
        ("MYR", "Malaysian ringgit"),
        ("MZN", "Mozambican metical"),
        ("NAD", "Namibian dollar"),
        ("NGN", "Nigerian naira"),
        ("NIO", "Nicaraguan córdoba"),
        ("NOK", "Norwegian krone"),
        ("NPR", "Nepalese rupee"),
        ("NZD", "New Zealand dollar"),
        ("OMR", "Omani rial"),
        ("PAB", "Panamanian balboa"),
        ("PEN", "Peruvian sol"),
        ("PGK", "Papua New Guinean kina"),
        ("PHP", "Philippine peso"),
        ("PKR", "Pakistani rupee"),
        ("PLN", "Polish zloty"),
        ("PYG", "Paraguayan guarani"),
        ("QAR", "Qatari riyal"),
        ("RON", "Romanian leu"),
        ("RSD", "Serbian dinar"),
        ("RUB", "Russian ruble"),
        ("RWF", "Rwandan franc"),
        ("SAR", "Saudi riyal"),
        ("SBD", "Solomon Islands dollar"),
        ("SCR", "Seychellois rupee"),
        ("SDG", "Sudanese pound"),
        ("SEK", "Swedish krona"),
        ("SGD", "Singapore dollar"),
        ("SHP", "Saint Helena pound"),
        ("SLL", "Sierra Leonean leone"),
        ("SOS", "Somali shilling"),
        ("SPL", "Seborga luigino"),
        ("SRD", "Surinamese dollar"),
        ("STD", "São Tomé and Príncipe dobra"),
        ("SVC", "Salvadoran colón"),
        ("SYP", "Syrian pound"),
        ("SZL", "Swazi lilangeni"),
        ("THB", "Thai baht"),
        ("TJS", "Tajikistani somoni"),
        ("TMT", "Turkmenistan manat"),
        ("TND", "Tunisian dinar"),
        ("TOP", "Tongan paʻanga"),
        ("TRY", "Turkish lira"),
        ("TTD", "Trinidad and Tobago dollar"),
        ("TVD", "Tuvaluan dollar"),
        ("TWD", "New Taiwan dollar"),
        ("TZS", "Tanzanian shilling"),
        ("UAH", "Ukrainian hryvnia"),
        ("UGX", "Ugandan shilling"),
        ("USD", "United States dollar"),
        ("UYU", "Uruguayan peso"),
        ("UZS", "Uzbekistani soʻm"),
        ("VEF", "Venezuelan bolívar"),
        ("VND", "Vietnamese đồng"),
        ("VUV", "Vanuatu vatu"),
        ("WST", "Samoan tālā"),
        ("XAF", "Central African CFA franc"),
        ("XCD", "Eastern Caribbean dollar"),
        ("XDR", "Special drawing rights"),
        ("XOF", "West African CFA franc"),
        ("XPF", "CFP franc"),
        ("YER", "Yemeni rial"),
        ("ZAR", "South African rand"),
        ("ZMW", "Zambian kwacha"),
        ("ZWD", "Zimbabwean dollar"),
    )

    # Source: https://en.wikipedia.org/wiki/List_of_cryptocurrencies
    cryptocurrencies = (
        ('AMP', "AMP"),
        ('AUR', "Auroracoin"),
        ('BC', "BlackCoin"),
        ('BTC', "Bitcoin"),
        ('BURST', "Burstcoin"),
        ('DASH', "Dash"),
        ('DOGE', "Dogecoin"),
        ('EMC', "Emercoin"),
        ('ETH', "Ethereum"),
        ('ETC', "Ethereum Classic"),
        ('GRC', "Gridcoin"),
        ('KOI', "Coinye"),
        ('LTC', "Litecoin"),
        ('MSC', "Omni"),
        ('MZC', "MazaCoin"),
        ('NMC', "Namecoin"),
        ('NXT', "Nxt"),
        ('POT', "PotCoin"),
        ('PPC', "Peercoin"),
        ('TIT', "Titcoin"),
        ('VTC', "Vertcoin"),
        ('XDN', "DigitalNote"),
        ('XMR', "Monero"),
        ('XPM', "Primecoin"),
        ('XRP', "Ripple"),
        ('ZEC', "Zcash"),
        ('STC', "SwiftCoin"),
        ('BCN', "Bytecoin"),
        ('FTH', "Feathercoin"),
        ('NEO', "NEO"),
        ('NEM', "XEM"),
        ('USDT', "Tether"),
        ('IOTA', "IOTA"),
        ('DRC', "Decred"),
        ('WAVES', "Waves Platform"),
        ('LSK', "Lisk"),
        ('ZCL', "Zclassic"),
        ('BCH', "Bitcoin Cash"),
        ('UBQ', "Ubiq"),
        ('EOS', "EOS.IO"),
        ('SRN', "Sirin Labs"),
        ('TRX', "TRON"),
        ('ADA', "Cardano"),
    )

    # List of currency symbols in Unicode, source: https://www.unicode.org/charts/beta/nameslist/n_20A0.html
    currency_symbols = {
        'AFN': '\u060B', 'ANG': '\u0192', 'ARS': '\u0024', 'AUD': '\u0024', 'AWG': '\u0192', 'BBD': '\u0024',
        'BDT': '\u09F3', 'BMD': '\u0024', 'BND': '\u0024', 'BOB': '\u0024', 'BRL': '\u0024', 'BSD': '\u0024',
        'BZD': '\u0024', 'CAD': '\u0024', 'CLP': '\u0024', 'CNY': '\u00A5', 'COP': '\u0024', 'CRC': '\u20A1',
        'CUP': '\u0024', 'CVE': '\u0024', 'DOP': '\u0024', 'EGP': '\u00A3', 'EUR': '\u20AC', 'FJD': '\u0024',
        'FKP': '\u00A3', 'GBP': '\u00A3', 'GHS': '\u20B5', 'GIP': '\u00A3', 'GYD': '\u0024', 'HKD': '\u0024',
        'HUF': '\u0192', 'IDR': '\u20A8', 'ILS': '\u20AA', 'INR': '\u20B9', 'IRR': '\uFDFC', 'JMD': '\u0024',
        'JPY': '\u00A5', 'KHR': '\u17DB', 'KPW': '\u20A9', 'KRW': '\u20A9', 'KYD': '\u0024', 'KZT': '\u20B8',
        'LAK': '\u20AD', 'LBP': '\u00A3', 'LKR': '\u20A8', 'LRD': '\u0024', 'MNT': '\u20AE', 'MOP': '\u0024',
        'MUR': '\u20A8', 'MXN': '\u0024', 'NAD': '\u0024', 'NGN': '\u20A6', 'NIO': '\u0024', 'NPR': '\u20A8',
        'NZD': '\u0024', 'OMR': '\uFDFC', 'PHP': '\u20B1', 'PKR': '\u20A8', 'PYG': '\u20B2', 'QAR': '\uFDFC',
        'RUB': '\u20BD', 'SAR': '\uFDFC', 'SBD': '\u0024', 'SDG': '\u00A3', 'SGD': '\u0024', 'SHP': '\u00A3',
        'SRD': '\u0024', 'SYP': '\u00A3', 'THB': '\u0E3F', 'TOP': '\u0024', 'TRY': '\u20BA', 'TTD': '\u0024',
        'TWD': '\u0024', 'UAH': '\u20B4', 'USD': '\u0024', 'UY': '\u0024', 'VND': '\u20AB', 'WST': '\u0024',
        'XCD': '\u0024', 'YER': '\uFDFC', 'ZWD': '\u0024',
    }

    price_formats = ["#.##", "%#.##", "%##.##", "%,###.##", "%#,###.##"]

    def currency(self):
        return self.random_element(self.currencies)

    def currency_code(self):
        return self.currency()[0]

    def currency_name(self):
        return self.currency()[1]

    def currency_symbol(self, code=None):
        """
        :example: $
        """
        if code is None:
            code = self.random_element(self.currency_symbols.keys())
        return self.currency_symbols[code]

    def cryptocurrency(self):
        return self.random_element(self.cryptocurrencies)

    def cryptocurrency_code(self):
        return self.cryptocurrency()[0]

    def cryptocurrency_name(self):
        return self.cryptocurrency()[1]

    def pricetag(self):
        return (
            self.random_element(self.currencies)[0]
            + "\N{no-break space}"
            + self.numerify(self.random_element(self.price_formats))
        )



import re

from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``tr_TR`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Turkey
    """

    license_formats = (
        '## ? ####',
        '## ? #####',
        '## ?? ###',
        '## ?? ####',
        '## ??? ##',
        '## ??? ###',
    )
    ascii_uppercase_turkish = 'ABCDEFGHIJKLMNOPRSTUVYZ'

    def license_plate(self) -> str:
        """Generate a license plate."""
        temp = re.sub(r'\?',
                      lambda x: self.random_element(self.ascii_uppercase_turkish),
                      self.random_element(self.license_formats))
        temp = temp.replace('##', '{:02d}', 1)
        temp = temp.format(self.random_element(range(1, 82)))
        return self.numerify(temp)



from .. import Provider as CurrencyProvider


class Provider(CurrencyProvider):

    price_formats = ["#,#0", "%#,#0", "%##,#0", "%.###,#0", "%#.###,#0"]

    def pricetag(self):
        return (
            self.numerify(self.random_element(self.price_formats))
            + "\N{no-break space}Kč"
        )



from .. import Provider as CurrencyProvider


class Provider(CurrencyProvider):
    # Format: (code, name)
    # See currency names in Russian: https://ru.wikipedia.org/wiki/Список_существующих_валют#Валюты
    currencies = (
        ("AED", "Дирхам ОАЭ"),
        ("AFN", "Афгани"),
        ("ALL", "Лек"),
        ("AMD", "Армянский драм"),
        ("ANG", "Нидерландский антильский гульден"),
        ("AOA", "Кванза"),
        ("ARS", "Аргентинское песо"),
        ("AUD", "Австралийский доллар"),
        ("AWG", "Арубанский флорин"),
        ("AZN", "Азербайджанский манат"),
        ("BAM", "Конвертируемая марка Боснии и Герцеговины"),
        ("BBD", "Барбадосский доллар"),
        ("BDT", "Така"),
        ("BGN", "Болгарский лев"),
        ("BHD", "Бахрейнский динар"),
        ("BIF", "Бурундийский франк"),
        ("BMD", "Бермудский доллар"),
        ("BND", "Брунейский доллар"),
        ("BOB", "Боливиано"),
        ("BRL", "Бразильский реал"),
        ("BSD", "Багамский доллар"),
        ("BTN", "Нгултрум"),
        ("BWP", "Пула"),
        ("BYR", "Белорусский рубль"),
        ("BZD", "Белизский доллар"),
        ("CAD", "Канадский доллар"),
        ("CDF", "Конголезский франк"),
        ("CHF", "Швейцарский франк"),
        ("CLP", "Чилийское песо"),
        ("CNY", "Юань"),
        ("COP", "Колумбийское песо"),
        ("CRC", "Коста-риканский колон"),
        ("CUC", "Кубанское конвертируемое песо"),
        ("CUP", "Кубанское песо"),
        ("CVE", "Эскудо Кабо-Верде"),
        ("CZK", "Чешская крона"),
        ("DJF", "Франк Джибути"),
        ("DKK", "Датская крона"),
        ("DOP", "Доминиканское песо"),
        ("DZD", "Алжирский динар"),
        ("EGP", "Египетский фунт"),
        ("ERN", "Накфа"),
        ("ETB", "Эфиопский быр"),
        ("EUR", "Евро"),
        ("FJD", "Доллар Фиджи"),
        ("FKP", "Фунт Фолклендских островов"),
        ("GBP", "Фунт стерлингов"),
        ("GEL", "Лари"),
        ("GGP", "Гернсийский фунт"),
        ("GHS", "Ганский седи"),
        ("GIP", "Гибралтарский фунт"),
        ("GMD", "Даласи"),
        ("GNF", "Гвинейский франк"),
        ("GTQ", "Кетсаль"),
        ("GYD", "Гайанский доллар"),
        ("HKD", "Гонконгский доллар"),
        ("HNL", "Лемпира"),
        ("HRK", "Хорватская куна"),
        ("HTG", "Гурд"),
        ("HUF", "Форинт"),
        ("IDR", "Индонезийская рупия"),
        ("ILS", "Новый израильский шекель"),
        ("NIS", "Новый израильский шекель"),
        ("IMP", "Фунт острова Мэн"),
        ("INR", "Индийская рупия"),
        ("IQD", "Иракский динар"),
        ("IRR", "Иранский риал"),
        ("ISK", "Исландская крона"),
        ("JEP", "Джерсийский фунт"),
        ("JMD", "Ямайский доллар"),
        ("JOD", "Иорданский динар"),
        ("JPY", "Иена"),
        ("KES", "Кенийский шиллинг"),
        ("KGS", "Сом"),
        ("KHR", "Риель"),
        ("KMF", "Франк Комор"),
        ("KPW", "Северокорейская вона"),
        ("KRW", "Южнокорейская вона"),
        ("KWD", "Кувейтский динар"),
        ("KYD", "Доллар Островов Кайман"),
        ("KZT", "Тенге"),
        ("LAK", "Кип"),
        ("LBP", "Ливийский фунт"),
        ("LKR", "Шри-ланкийская рупия"),
        ("LRD", "Либерийский доллар"),
        ("LSL", "Лоти"),
        ("LTL", "Литовский лит"),
        ("LYD", "Ливийский динар"),
        ("MAD", "Марокканский дирхам"),
        ("MDL", "Молдавский лей"),
        ("MGA", "Малагасийский ариари"),
        ("MKD", "Денар"),
        ("MMK", "Кьят"),
        ("MNT", "Тугрик"),
        ("MOP", "Патака"),
        ("MRO", "Угия"),
        ("MUR", "Маврикийская рупия"),
        ("MVR", "Рувия"),
        ("MWK", "Квача"),
        ("MXN", "Мексиканское песо"),
        ("MYR", "Малайзийский ринггит"),
        ("MZN", "Мозамбикский метикал"),
        ("NAD", "Доллар Намибии"),
        ("NGN", "Найра"),
        ("NIO", "Кордоба"),
        ("NOK", "Норвежская крона"),
        ("NPR", "Непальская рупия"),
        ("NZD", "Новозеландский доллар"),
        ("OMR", "Оманский риал"),
        ("PAB", "Бальбоа"),
        ("PEN", "Соль"),
        ("PGK", "Кина"),
        ("PHP", "Филиппинское песо"),
        ("PKR", "Пакистанская рупия"),
        ("PLN", "Злотый"),
        ("PYG", "Гуарани"),
        ("QAR", "Катарский риал"),
        ("RON", "Румынский лей"),
        ("RSD", "Сербский динар"),
        ("RUB", "Российский рубль"),
        ("RWF", "Франк Руанды"),
        ("SAR", "Саудовский риял"),
        ("SBD", "Доллар Соломоновых Островов"),
        ("SCR", "Сейшельская рупия"),
        ("SDG", "Суданский фунт"),
        ("SEK", "Шведская крона"),
        ("SGD", "Сингапурский доллар"),
        ("SHP", "Фунт Святой Елены"),
        ("SLL", "Леоне"),
        ("SOS", "Сомалийский шиллинг"),
        ("SPL", "Луиджино"),
        ("SRD", "Суринамский доллар"),
        ("STD", "Добра"),
        ("SVC", "Сальвадорский колон"),
        ("SYP", "Сирийский фунт"),
        ("SZL", "Лилангени"),
        ("THB", "Бат"),
        ("TJS", "Сомони"),
        ("TMT", "Новый туркменский манат"),
        ("TND", "Тунисский динар"),
        ("TOP", "Паанга"),
        ("TRY", "Турецкая лира"),
        ("TTD", "Доллар Тринидада и Тобаго"),
        ("TVD", "Доллар Тувалу"),
        ("TWD", "Новый тайваньский доллар"),
        ("TZS", "Танзанийский шиллинг"),
        ("UAH", "Гривна"),
        ("UGX", "Угандийский шиллинг"),
        ("USD", "Доллар США"),
        ("UYU", "Уругвайское песо"),
        ("UZS", "Узбекский сум"),
        ("VEF", "Суверенный боливар"),
        ("VND", "Донг"),
        ("VUV", "Вату"),
        ("WST", "Тала"),
        ("XAF", "Франк КФА ВЕАС"),
        ("XCD", "Восточно-карибский доллар"),
        ("XDR", "СДР"),
        ("XOF", "Франк КФА ВСЕАО"),
        ("XPF", "Франк КФП"),
        ("YER", "Йеменский риал"),
        ("ZAR", "Рэнд"),
        ("ZMW", "Замбийская квача"),
        ("ZWD", "Доллар Зимбабве"),
    )

    price_formats = ["#,##", "%#,##", "%##,##", "%.###,##", "%#.###,##"]

    def pricetag(self):
        return (
            self.numerify(self.random_element(self.price_formats))
            + "\N{no-break space}\N{cyrillic small letter er}."
        )



from .. import Provider as SsnProvider


class Provider(SsnProvider):
    ssn_formats = ("###.####.####.##",)

    def ssn(self):
        """
        Returns a 13 digits Swiss SSN named AHV (German) or
                                            AVS (French and Italian)
        See: http://www.bsv.admin.ch/themen/ahv/00011/02185/
        """
        def _checksum(digits):
            evensum = sum(digits[:-1:2])
            oddsum = sum(digits[1::2])
            return (10 - ((evensum + oddsum * 3) % 10)) % 10

        digits = [7, 5, 6]
        # create an array of first 9 elements initialized randomly
        digits += self.generator.random.sample(range(10), 9)
        # determine the last digit to make it qualify the test
        digits.append(_checksum(digits))
        # repeat steps until it does qualify the test

        digits = ''.join([str(d) for d in digits])
        ssn = digits[:3] + '.' \
                         + digits[3:7] + '.' \
                         + digits[7:11] + '.' \
                         + digits[11:]
        return ssn

    def vat_id(self):
        """
        :return: Swiss UID number
        """
        def _checksum(digits):
            code = ['8', '6', '4', '2', '3', '5', '9', '7']
            remainder = 11-(sum(map(lambda x, y: int(x) * int(y), code, digits)) % 11)
            if remainder == 10:
                return 0
            elif remainder == 11:
                return 5
            return remainder

        vat_id = self.numerify('########')
        return 'CHE' + vat_id + str(_checksum(vat_id))



from ..de_CH import Provider as DeChBankProvider


class Provider(DeChBankProvider):
    """Implement bank provider for ``fr_CH`` locale.

    There is no difference from the ``de_CH`` implementation.
    """
    pass
